import os
import numpy as np

from PyQt4 import QtCore
from PyQt4 import QtGui
import OpenGL.GL as GL

N_PREFETCH = 10

checkBoxStyleStr = """
QCheckBox::indicator:checked {
background:black;
border-image: url(%s);
}
QCheckBox::indicator:unchecked {
background:black;
color:white;
border-image: url(%s);}

QToolTip { color:white;}
"""
checkNormalBoxStyleStr = """
QCheckBox::indicator:checked {
}
QCheckBox::indicator:unchecked {
background:black;
color:white;}
QToolTip { color:white;}
"""

checkBoxTristateStyleStr = """
QCheckBox::indicator:unchecked {
background:black;
color:white;
border-image: url(%s);}

QCheckBox::indicator:indeterminate {
background:black;
color:white;
border-image: url(%s);}

QCheckBox::indicator:checked {
background:black;
color:white;
border-image: url(%s);
}
QToolTip { color:white;}
"""


def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


def createStandardButton(parent,fName = None,method = None, width = 24, tooltip = ""):
    but = QtGui.QPushButton("",parent)
    but.setStyleSheet("background-color: black;color:black;color:lightgrey;")
    if fName:
        but.setIcon(QtGui.QIcon(fName))
    but.setIconSize(QtCore.QSize(width,width))
    if method:
        but.clicked.connect(method)
    but.setMaximumWidth(width)
    but.setMaximumHeight(width)
    if tooltip:
        but.setToolTip(tooltip)
    return but


def createImageCheckbox(parent, img1=None, img2 = None, tooltip =""):
    check = QtGui.QCheckBox("",parent)
    checkStr = checkBoxStyleStr%(absPath(img1),absPath(img2))
    if os.name =="nt":
        checkStr = checkStr.replace("\\","/")

    check.setStyleSheet(checkStr)
    if tooltip:
        check.setToolTip(tooltip)
    return check

def createStandardCheckbox(parent,  tooltip =""):
    check = QtGui.QCheckBox("",parent)
    checkStr = checkNormalBoxStyleStr
    if os.name =="nt":
        checkStr = checkStr.replace("\\","/")

    check.setStyleSheet(checkStr)
    check.setToolTip(tooltip)
    return check

def createTristateCheckbox(parent, img1=None , img2 = None,img3 = None, tooltip = ""):
    check = QtGui.QCheckBox("",parent)
    check.setTristate()

    checkStr = checkBoxTristateStyleStr%(absPath(img1),absPath(img2),absPath(img3))
    if os.name =="nt":
        checkStr = checkStr.replace("\\","/")

    check.setStyleSheet(checkStr)
    check.setToolTip(tooltip)
    return check





def fillTexture2d(data,tex = None):
    """ data.shape == (Ny,Nx)
          file texture with GL_RED
        data.shape == (Ny,Nx,3)
          file texture with GL_RGB

        if tex == None, returns a new created texture
    """

    if tex is None:
        tex = GL.glGenTextures(1)

    GL.glBindTexture(GL.GL_TEXTURE_2D, tex)
    GL.glPixelStorei(GL.GL_UNPACK_ALIGNMENT,1)
    GL.glTexParameterf (GL.GL_TEXTURE_2D,
                     GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
    GL.glTexParameterf (GL.GL_TEXTURE_2D,
                     GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)

    GL.glTexParameterf (GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP_TO_EDGE)
    GL.glTexParameterf (GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP_TO_EDGE)
    # GL.glTexParameterf (GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP)
    # GL.glTexParameterf (GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP)

    if data.ndim == 2:
        Ny,Nx = data.shape
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, Nx, Ny,
                     0, GL.GL_RED, GL.GL_FLOAT, data.astype(np.float32))

    elif data.ndim == 3 and data.shape[2]==3:
        Ny,Nx = data.shape[:2]
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGB, Nx, Ny,
                         0, GL.GL_RGB, GL.GL_FLOAT, data.astype(np.float32))

    elif data.ndim == 3 and data.shape[2]==4:
        Ny,Nx = data.shape[:2]
        GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA, Nx, Ny,
                         0, GL.GL_RGBA, GL.GL_FLOAT, data.astype(np.float32))

    else:
        raise Exception("data format not supported! \ndata.shape should be either (Ny,Nx) or (Ny,Nx,3)")
    return tex

def arrayFromImage(fName):
    """converts png image to float32 array
    returns an array of shape [w,h,3]
    """
    try:
        img = QtGui.QImage(fName).convertToFormat(QtGui.QImage.Format_RGB32)


        Nx, Ny = img.width(),img.height()
        tmp = img.bits().asstring(img.numBytes())
        arr = np.frombuffer(tmp, np.uint8).reshape((Ny,Nx,4))
        arr = arr.astype(np.float32)/np.amax(arr)
        return arr[:,:,:-1][:,:,::-1]
    except Exception as e:
        print e
        print "could not load image %s"%fName
        return np.zeros((10,100,3),np.float32)





def slice_coords(relPos,dim):

    if dim ==0:
        coords = np.array([[0,-1,-1],[0,1,-1],[0,1,1],
                    [0,1,1],[0,-1,1],[0,-1,-1]],dtype=np.float32)

    elif dim ==1:
        coords = np.array([[-1,0,-1],[1,0,-1],[1,0,1],
                    [1,0,1],[-1,0,1],[-1,0,-1]],dtype=np.float32)

    elif dim ==2:
        coords = np.array([[-1,-1,0],[1,-1,0],[1,1,0],
                    [1,1,0],[-1,1,0],[-1,-1,0]],dtype=np.float32)


    coords[:,dim] = -1.+2*relPos

    return coords


def create_quad_coords(bounds = [-1,1.,-1,1]):
    x1,x2,y1,y2 = bounds
    return  np.array([[x1,y1],
                   [x2,y1],
                       [x2,y2],
                       [x2,y2],
                       [x1,y2],
                       [x1,y1]])



def create_cube_coords(bounds = [-1,1.,-1,1,-1,1]):
    x1,x2,y1,y2,z1,z2 = bounds
    return np.array([[x2, y2, z2], [x1, y2, z2],
                     [x1, y2, z2], [x1, y1, z2],
                     [x1, y1, z2], [x2, y1, z2],
                     [x2, y1, z2], [x2, y2, z2],

                     [x2, y2, z1], [x1, y2, z1],
                     [x1, y2, z1], [x1, y1, z1],
                     [x1, y1, z1], [x2, y1, z1],
                     [x2, y1, z1], [x2, y2, z1],

                     [x2, y2, z2], [x2, y2, z1],
                     [x1, y2, z2], [x1, y2, z1],
                     [x1, y1, z2], [x1, y1, z1],
                     [x2, y1, z2], [x2, y1, z1],
                     ])



def create_sphere_coords(rx,ry,rz,Nphi=50, Ntheta=30, return_normals = False):
    ts = np.arccos(np.linspace(-1.,1.,Ntheta))
    ps = np.linspace(0,2.*np.pi,Nphi+1)

    T,P = np.meshgrid(ts,ps, indexing = "ij")

    xs = np.array([rx*np.cos(P)*np.sin(T),ry*np.sin(P)*np.sin(T),rz*np.cos(T)])

    coords = []
    normals = []
    for i in range(len(ts)-1):
        for j in range(len(ps)-1):
            coords.append(xs[:,i,j])
            coords.append(xs[:,i+1,j])
            coords.append(xs[:,i+1,j+1])
            coords.append(xs[:,i,j])
            coords.append(xs[:,i+1,j+1])
            coords.append(xs[:,i,j+1])

            #FIXME, wrong for rx != ry ....
            normals.append(1.*xs[:,i,j]/rx)
            normals.append(1.*xs[:,i+1,j]/rx)
            normals.append(1.*xs[:,i+1,j+1]/rx)
            normals.append(1.*xs[:,i,j]/rx)
            normals.append(1.*xs[:,i,j+1]/rx)
            normals.append(1.*xs[:,i+1,j+1]/rx)
    if return_normals:
        return np.array(coords), np.array(normals)
    else:
        return np.array(coords)



if __name__ == '__main__':
    c =  create_sphere_coords(.8,10,10)
    



vertShaderTex ="""
attribute vec2 position;
attribute vec2 texcoord;
varying vec2 mytexcoord;

void main()
{
    gl_Position = vec4(position, 0., 1.0);
    mytexcoord = texcoord;
}
"""

fragShaderTex = """
uniform sampler2D texture;
uniform sampler2D texture_LUT;
varying vec2 mytexcoord;

void main()
{
  vec4 col = texture2D(texture,mytexcoord);

  vec4 lut = texture2D(texture_LUT,col.xy);

  gl_FragColor = vec4(lut.xyz,col.x);
//  gl_FragColor.w = 1.0*length(gl_FragColor.xyz);

}
"""

vertShaderCube ="""
attribute vec3 position;
uniform mat4 mvpMatrix;

void main()
{
  vec3 pos = position;
  gl_Position = mvpMatrix *vec4(pos, 1.0);

}
"""

fragShaderCube = """
void main()
{
  gl_FragColor = vec4(1.,1.,1.,.6);
}
"""



"""
author: Martin Weigert
email: mweigert@mpi-cbg.de
"""

import logging
logger = logging.getLogger(__name__)



from PyQt4 import QtCore

from spimagine.utils.transform_matrices import *
from spimagine.utils.quaternion import Quaternion

import numpy as np

from spimagine.models.keyframe_model import TransformData


class TransformModel(QtCore.QObject):
    _maxChanged = QtCore.pyqtSignal(float)
    _minChanged = QtCore.pyqtSignal(float)
    
    _gammaChanged = QtCore.pyqtSignal(float)
    _boxChanged = QtCore.pyqtSignal(int)

    _isoChanged = QtCore.pyqtSignal(bool)

    _perspectiveChanged = QtCore.pyqtSignal(int)
    # _rotationChanged = QtCore.pyqtSignal(float,float,float,float)
    _rotationChanged = QtCore.pyqtSignal()

    _translateChanged = QtCore.pyqtSignal(float,float,float)
    _slicePosChanged =  QtCore.pyqtSignal(int)
    _sliceDimChanged =  QtCore.pyqtSignal(int)
    _boundsChanged = QtCore.pyqtSignal(float,float,float,float,float,float)

    _transformChanged = QtCore.pyqtSignal()
    _stackUnitsChanged = QtCore.pyqtSignal(float,float,float)

    _alphaPowChanged = QtCore.pyqtSignal(float)

    def __init__(self):
        super(TransformModel,self).__init__()
        self.reset()

    def _update_value(self,name,newval):
        """update self.name to newval and returns True if the
        new value indeed was different than the new"""
        if not hasattr(self,name):
            setattr(self,name,newval)
            return True

        oldval = getattr(self,name)
        is_equal = True

        if isinstance(oldval,np.ndarray):
            is_equal = np.array_equal(oldval,newval)
        elif isinstance(oldval,Quaternion):
            is_equal = np.array_equal(oldval.data,newval.data)
        else:
            is_equal = (oldval == newval)

        if not is_equal:
            setattr(self,name,newval)

        return not is_equal


    def setModel(self,dataModel):
        self.dataModel = dataModel

    def reset(self,minVal = 0., maxVal = 256.,stackUnits=None):
        logger.debug("reset:  min = %s max = %s"%(minVal,maxVal))
        self.dataPos = 0
        self.slicePos = 0
        self.sliceDim = 0
        self.zoom = 1.
        self.setIso(False)
        self.isPerspective = True
        self.setPerspective()
        self.setValueScale(minVal,maxVal)
        self.setGamma(1.)
        self.setAlphaPow(0)
        self.setBox(True)

        self.setOccStrength()
        self.setOccRadius()
        self.setOccNPoints()

        self.eye_dist_proj = 0
        self.eye_dist_cam = 0
        if not hasattr(self,"isSlice"):
            self.setShowSlice(False)

        if not stackUnits:
            stackUnits = [.1,.1,.1]
        self.setStackUnits(*stackUnits)
        self.center()


    def setIso(self,isIso):
        logger.debug("setting Iso %s"%isIso)
        if self._update_value("isIso",isIso):
            self._isoChanged.emit(isIso)
            self._transformChanged.emit()

    def setOccStrength(self, occ_strength = .15):
        if self._update_value("occ_strength",occ_strength):
            self._transformChanged.emit()

    def setOccRadius(self, val = 21):
        if self._update_value("occ_radius",val):
            self._transformChanged.emit()

    def setOccNPoints(self, val = 31):
        if self._update_value("occ_n_points",val):
            self._transformChanged.emit()

    def center(self):
        self.quatRot = Quaternion()
        self.cameraZ = 5.
        self.zoom  = 1.
        self.scaleAll = 1.
        self.setBounds(-1,1.,-1,1,-1,1)
        self.setTranslate(0,0,0)

        self.update()
        self._transformChanged.emit()

    def setTranslate(self,x,y,z):
        newtrans = np.array([x,y,z])

        if self._update_value("translate",newtrans):
            self._translateChanged.emit(x,y,z)
            self._transformChanged.emit()

    def addTranslate(self,dx,dy,dz):
        self.translate = self.translate + np.array([dx,dy,dz])
        self._translateChanged.emit(*self.translate)
        self._transformChanged.emit()

    def setBounds(self,x1,x2,y1,y2,z1,z2):
        self.bounds = np.array([x1,x2,y1,y2,z1,z2])
        self._boundsChanged.emit(x1,x2,y1,y2,z1,z2)
        self._transformChanged.emit()

    def setShowSlice(self,isSlice=True):
        self.isSlice = isSlice
        self._transformChanged.emit()

    def setSliceDim(self,dim):
        logger.debug("setSliceDim(%s)",dim)
        if dim>= 0 and dim<3:
            self.sliceDim = dim
            self._sliceDimChanged.emit(dim)
            self._transformChanged.emit()
        else:
            raise ValueError("dim should be in [0,1,2]!")

    def setSlicePos(self,pos):
        logger.debug("setSlicePos(%s)",pos)
        self.slicePos = pos
        self._slicePosChanged.emit(pos)
        self._transformChanged.emit()

    def setPos(self,pos):
        logger.debug("setPos(%s)",pos)
        self.dataPos = pos
        self.dataModel.setPos(pos)
        self._transformChanged.emit()

    def setGamma(self, gamma):
        logger.debug("setGamma(%s)",gamma)

        self.gamma = gamma
        self._gammaChanged.emit(self.gamma)
        self._transformChanged.emit()


    def setAlphaPow(self, alphaPow):
        logger.debug("setAlphaPow(%s)",alphaPow)
        self.alphaPow = alphaPow
        self._alphaPowChanged.emit(self.alphaPow)
        self._transformChanged.emit()

    def setValueScale(self,minVal,maxVal):
        logger.debug("set scale to %s,%s"%(minVal, maxVal))

        self.setMin(minVal)
        self.setMax(maxVal)

    def setMin(self,minVal):
        self.minVal = max(1.e-6,minVal)
        logger.debug("set min to %s"%(self.minVal))

        self._minChanged.emit(self.minVal)
        self._transformChanged.emit()

    def setMax(self,maxVal):
        self.maxVal = maxVal
        
        logger.debug("set max to %s"%(self.maxVal))

        self._maxChanged.emit(self.maxVal)
        self._transformChanged.emit()

    def setStackUnits(self,px,py,pz):
        self.stackUnits = px,py,pz
        self._stackUnitsChanged.emit(px,py,pz)
        self._transformChanged.emit()

    def setBox(self,isBox = True):
        self.isBox = isBox
        self._boxChanged.emit(isBox)
        self._transformChanged.emit()

    def setZoom(self,zoom = 1.):
        self.zoom = np.clip(zoom,.5,2)
        self.update()
        self._transformChanged.emit()


    def addRotation(self, angle, x, y, z):
        q = Quaternion(np.cos(angle),np.sin(angle)*x,np.sin(angle)*y,np.sin(angle)*z)
        self.setQuaternion(self.quatRot * q)

    def setRotation(self,angle,x,y,z):
        self.setQuaternion(Quaternion(np.cos(angle),np.sin(angle)*x,np.sin(angle)*y,np.sin(angle)*z))

    def setQuaternion(self,quat):
        logger.debug("set quaternion to %s",quat.data)
        self.quatRot = Quaternion.copy(quat)
        self._rotationChanged.emit()
        self._transformChanged.emit()


    def setEyeDistProj(self,eye_dist_proj = 0):
        self.eye_dist_proj = eye_dist_proj
        self.update()
        print self.eye_dist_proj
        self._transformChanged.emit()

    def setEyeDistCam(self,eye_dist_cam = 0.):
        self.eye_dist_cam = eye_dist_cam
        print self.eye_dist_cam
        self.update()
        self._transformChanged.emit()

    def update(self):
        if self.isPerspective:
            self.cameraZ = 4*(1-np.log(self.zoom)/np.log(2.))
            self.scaleAll = 1.
        else:
            self.cameraZ = 0.
            self.scaleAll = 2.5**(self.zoom-1.)

    def setPerspective(self, isPerspective = True):
        self.isPerspective = isPerspective
        if isPerspective:
            self.projection = mat4_perspective(60.,1.,.1,10)
        else:
            self.projection = mat4_ortho(-2.,2.,-2.,2.,-1.5,1.5)

        self.update()
        self._perspectiveChanged.emit(isPerspective)
        self._transformChanged.emit()

    def getProjection(self):
        return self.projection

    def getModelView(self):
        """ returns the modelview with the added internal scale from the rendered volume
        this should be used when drawing standard opengl primitives with the same trasnformation as
        the rendered model"""

        modelView = self.getUnscaledModelView()

        #scale the interns
        if hasattr(self,"dataModel"):
            Nz,Ny,Nx = self.dataModel.size()[1:]
            dx,dy,dz = self.stackUnits
            maxDim = max(d*N for d,N in zip([dx,dy,dz],[Nx,Ny,Nz]))
            mScale =  mat4_scale(1.*dx*Nx/maxDim,1.*dy*Ny/maxDim,1.*dz*Nz/maxDim)
            modelView = np.dot(modelView,mScale)

        return modelView


    def getUnscaledModelView(self):
        view  = mat4_translate(0,0,-self.cameraZ)

        model = mat4_scale(*[self.scaleAll]*3)
        model = np.dot(model,self.quatRot.toRotation4())
        model = np.dot(model,mat4_translate(*self.translate))

        # return model
        return np.dot(view,model)

    def fromTransformData(self,transformData):
        self.setQuaternion(transformData.quatRot)
        self.setZoom(transformData.zoom)
        self.setPos(transformData.dataPos)
        self.setBounds(*transformData.bounds)
        self.setBox(transformData.isBox)
        self.setIso(transformData.isIso)

        self.setAlphaPow(transformData.alphaPow)
        self.setTranslate(*transformData.translate)
        self.setValueScale(transformData.minVal,transformData.maxVal)
        # self.setGamma(transformData.gamma)

    def toTransformData(self):
        return TransformData(quatRot = self.quatRot, zoom = self.zoom,
                             dataPos = self.dataPos,
                             minVal = self.minVal,
                             maxVal = self.maxVal,
                             gamma= self.gamma,
                             translate = self.translate,
                             bounds = self.bounds,
                             isBox = self.isBox,
                             isIso = self.isIso,
                             alphaPow = self.alphaPow)




#!/usr/bin/env python

"""

The 2d slice widget


author: Martin Weigert
email: mweigert@mpi-cbg.de
"""


import logging

logger = logging.getLogger(__name__)




import sys
import os
from PyQt4 import QtCore
from PyQt4 import QtGui
from PyQt4 import QtOpenGL

from OpenGL.GL import *
from OpenGL.GL import shaders

import spimagine


from spimagine.volumerender.volume_render import VolumeRenderer
from spimagine.utils.transform_matrices import *
from spimagine.models.data_model import DataModel
from spimagine.models.transform_model import TransformModel
from spimagine.utils.quaternion import Quaternion
from spimagine.gui.gui_utils import *

from numpy import *
import numpy as np


# on windows numpy.linalg.inv crashes without notice, so we have to import scipy.linalg
if os.name == "nt":
    from scipy import linalg


import time

vertShaderTex ="""
attribute vec2 position;
attribute vec2 texcoord;
varying vec2 mytexcoord;



void main()
{
    gl_Position = vec4(position, 0., 1.0);
    mytexcoord = texcoord;
}
"""

fragShaderTex = """
uniform sampler2D texture;
uniform sampler2D texture_LUT;
varying vec2 mytexcoord;


void main()
{
  vec4 col = texture2D(texture,mytexcoord);

  vec4 lut = texture2D(texture_LUT,col.xy);

  gl_FragColor = vec4(lut.xyz,1.);

//  gl_FragColor.w = 1.0*length(gl_FragColor.xyz);

}
"""



def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)




class GLSliceWidget(QtOpenGL.QGLWidget):
    _dataModelChanged = QtCore.pyqtSignal()

    def __init__(self, parent=None,**kwargs):
        logger.debug("init")

        super(GLSliceWidget,self).__init__(parent,**kwargs)

        self.renderUpdate = True
        self.parent= parent

        self.setAcceptDrops(True)

        self.texture_LUT = None
        self.setTransform(TransformModel())

        self.renderTimer = QtCore.QTimer(self)
        self.renderTimer.setInterval(50)
        self.renderTimer.timeout.connect(self.onRenderTimer)
        self.renderTimer.start()

        self.dataModel = None

        self.dataPos = 0
        self.slicePos = 0

        self.zoom_fac = 1.
        self.zoom_x = 0.5
        self.zoom_y = 0.5
        # the center in tex coords
        self.zoom_cx = 0.5
        self.zoom_cy = 0.5


        # self.refresh()



    def setModel(self,dataModel):
        logger.debug("setModel")

        self.dataModel = dataModel

        if self.dataModel:
            self.transform.setModel(dataModel)

            self.dataModel._dataSourceChanged.connect(self.dataSourceChanged)
            self.dataModel._dataPosChanged.connect(self.dataPosChanged)
            self._dataModelChanged.connect(self.dataModelChanged)
            self._dataModelChanged.emit()


    def setTransform(self, transform):
        self.transform = transform
        self.transform._transformChanged.connect(self.refresh)


    def dragEnterEvent(self, event):
        if event.mimeData().hasUrls():
            event.accept()
        else:
            event.ignore()

    def dropEvent(self, event):
        for url in event.mimeData().urls():
            path = url.toLocalFile().toLocal8Bit().data()
            if self.dataModel:
                self.dataModel.loadFromPath(path, prefetchSize = self.N_PREFETCH)
            else:
                self.setModel(DataModel.fromPath(path, prefetchSize = self.N_PREFETCH))


    def set_colormap(self,name):
        """arr should be of shape (N,3) and gives the rgb components of the colormap"""

        try:
            arr = spimagine.config.__COLORMAPDICT__[name]
            self.makeCurrent()
            self.texture_LUT = fillTexture2d(arr.reshape((1,)+arr.shape),self.texture_LUT)
        except:
            print "could not load colormap %s"%name


    def set_colormap_rgb(self,color=[1.,1.,1.]):
        self._set_colormap_array(outer(linspace(0,1.,255),np.array(color)))

    def _set_colormap_array(self,arr):
        """arr should be of shape (N,3) and gives the rgb components of the colormap"""


        self.makeCurrent()
        self.texture_LUT = fillTexture2d(arr.reshape((1,)+arr.shape),self.texture_LUT)
        self.refresh()

        
    def initializeGL(self):

        self.resized = True

        self.output = zeros((100,100))

        logger.debug("initializeGL")

        self.programTex = QtOpenGL.QGLShaderProgram()
        self.programTex.addShaderFromSourceCode(QtOpenGL.QGLShader.Vertex,vertShaderTex)
        self.programTex.addShaderFromSourceCode(QtOpenGL.QGLShader.Fragment, fragShaderTex)
        self.programTex.link()
        self.programTex.bind()
        logger.debug("GLSL programTex log:%s",self.programTex.log())

        glClearColor(0,0,0,1.)

        self.texture = None


        self.quadCoord = np.array([[-1.,-1.,0.],
                           [1.,-1.,0.],
                           [1.,1.,0.],
                           [1.,1.,0.],
                           [-1.,1.,0.],
                           [-1.,-1.,0.]])

        self.quadCoordTex = np.array([[0,0],
                           [1.,0.],
                           [1.,1.],
                           [1.,1.],
                           [0,1.],
                           [0,0]])

        self.set_colormap(spimagine.config.__DEFAULTCOLORMAP__)

        glDisable(GL_DEPTH_TEST)
        glEnable( GL_BLEND )

        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)



    def dataModelChanged(self):
        if self.dataModel:
            self.transform.reset(amin(self.dataModel[0]),
                                 amax(self.dataModel[0]),
                                 self.dataModel.stackUnits())

            self.refresh()


    def dataSourceChanged(self):
        self.transform.reset(amin(self.dataModel[0]),
                             amax(self.dataModel[0]),
                             self.dataModel.stackUnits())
        self.refresh()



    def dataPosChanged(self,pos):
        self.dataPos = pos
        self.refresh()


    def refresh(self):
        # if self.parentWidget() and self.dataModel:
        #     self.parentWidget().setWindowTitle("SpImagine %s"%self.dataModel.name())
        self.renderUpdate = True

    def resizeGL(self, width, height):

        height = max(10,height)

        self.width , self.height = width, height

        self.resized = True

        self.resetViewPort()


    def getDataWidthHeight(self):
        if not self.dataModel:
            return 1,1

        dim = array(self.dataModel.size()[1:])[::-1].astype(np.float32)

        dim *= array(self.transform.stackUnits)

        if self.transform.sliceDim==0:
            dim = dim[[2,1]]
        elif self.transform.sliceDim==1:
            dim = dim[[0,2]]
        elif self.transform.sliceDim==2:
            dim = dim[[0,1]]

        w,h = dim[0],dim[1]



        # fac = 1.*min(self.width,self.height)/max(w,h)

        fac = min(1.*self.width/w,1.*self.height/h)


        return int(fac*w),int(fac*h)


    def resetViewPort(self):
        w,h = self.getDataWidthHeight()


        glViewport((self.width-w)/2,(self.height-h)/2,w,h)



    def paintGL(self):

        self.makeCurrent()

        if not glCheckFramebufferStatus(GL_FRAMEBUFFER)==GL_FRAMEBUFFER_COMPLETE:
            return

        #hack
        if self.resized:
            self.resetViewPort()


        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        if self.dataModel:
            # Draw the render texture
            self.programTex.bind()

            self.texture = fillTexture2d(self.output,self.texture)


            glEnable(GL_TEXTURE_2D)
            glDisable(GL_DEPTH_TEST)

            self.programTex.enableAttributeArray("position")
            self.programTex.enableAttributeArray("texcoord")
            self.programTex.setAttributeArray("position", self.quadCoord)
            self.programTex.setAttributeArray("texcoord",
                            self.tex_coords_from_xyzoom(self.zoom_x,self.zoom_y,self.zoom_fac))


            glActiveTexture(GL_TEXTURE0)
            glBindTexture(GL_TEXTURE_2D, self.texture)
            self.programTex.setUniformValue("texture",0)

            glActiveTexture(GL_TEXTURE1)
            glBindTexture(GL_TEXTURE_2D, self.texture_LUT)
            self.programTex.setUniformValue("texture_LUT",1)




            glDrawArrays(GL_TRIANGLES,0,len(self.quadCoord))



    def render(self):
        logger.debug("render")
        if self.dataModel:
            if self.transform.sliceDim==0:
                out = fliplr(self.dataModel[self.transform.dataPos][:,:,self.transform.slicePos].T)
            elif self.transform.sliceDim==1:
                out = self.dataModel[self.transform.dataPos][:,self.transform.slicePos,:]
            elif self.transform.sliceDim==2:
                out = self.dataModel[self.transform.dataPos][self.transform.slicePos,:,:]

            self.output = (1.*(out-self.transform.minVal)/(self.transform.maxVal-self.transform.minVal))**self.transform.gamma

            logger.debug("render: output range = %s"%([amin(self.output),amax(self.output)]))


    def saveFrame(self,fName):
        """FIXME: scaling behaviour still hast to be implemented (e.g. after setGamma)"""
        logger.info("saving frame as %s", fName)

        self.render()
        self.paintGL()
        glFlush()
        self.grabFrameBuffer().save(fName)


    def onRenderTimer(self):
        if self.renderUpdate:
            self.render()
            self.renderUpdate = False
            self.updateGL()

    def tex_coords_from_xyzoom(self,x0,y0,zoom):
        """returns array of texccords corners when zoomed
        in onto x0,y0 \in [0,1]"""

        q0 = create_quad_coords([0,1,0,1])
        q1 = create_quad_coords([x0,x0,y0,y0])

        return zoom*q0+(1.-zoom)*q1

    # def tex_coords_from_xyzoom(self,x0,y0,zoom):
    #     """returns array of texccords corners given a zoom
    #     and the tex center coords"""
    #
    #     q0 = create_quad_coords([0,1,0,1])
    #     q1 = create_quad_coords([x0,x0,y0,y0])
    #
    #     return zoom*q0+(1.-zoom)*q1

    def getRelativeCoords(self,x0,y0):
        w, h = self.getDataWidthHeight()

        x = 2.*(x0-.5*(self.width-w))/w-1
        y = 2.*(y0-.5*(self.height-h))/h-1

        x = (x0-.5*(self.width-w))/w
        y = 1-(y0-.5*(self.height-h))/h

        return x,y

    def mousePressEvent(self, event):
        super(GLSliceWidget, self).mousePressEvent(event)

        if event.buttons() == QtCore.Qt.LeftButton:
            self._x0, self._y0 = self.getRelativeCoords(event.x(),event.y())


    def mouseMoveEvent(self, event):

        if event.buttons() == QtCore.Qt.LeftButton:
            x, y = self.getRelativeCoords(event.x(),event.y())
            #print self._x0, x
            self.zoom_x += self.zoom_fac*(self._x0 - x)
            self.zoom_y += self.zoom_fac*(self._y0 - y)
            self._x0, self._y0 = x,y



            self.zoom_x , self.zoom_y = clip(self.zoom_x ,0,1), clip(self.zoom_y,0,1)

        self.refresh()


    def wheelEvent(self, event):
        # get the zoom factor

        # print self.zoom_x,self.zoom_y
        # x, y = self.getRelativeCoords(event.x(),event.y())
        # self.zoom_x , self.zoom_y = clip(x ,0,1), clip(y,0,1)



        self.zoom_fac *=  1.4**(-event.delta()/1000.)

        self.zoom_fac = clip(self.zoom_fac,0,1.)

        self.refresh()



class SliceWidget(QtGui.QWidget):

    def __init__(self, parent = None,**kwargs):
        super(SliceWidget,self).__init__(parent,**kwargs)

        self.myparent = parent

        self.glSliceWidget = GLSliceWidget(self)



        self.checkSlice = createTristateCheckbox(self,absPath("images/icon_x.png"),
                                                 absPath("images/icon_y.png"),
                                                 absPath("images/icon_z.png"))

        self.sliderSlice = QtGui.QSlider(QtCore.Qt.Horizontal,self)
        self.sliderSlice.setTickPosition(QtGui.QSlider.TicksBothSides)
        self.sliderSlice.setTickInterval(1)
        self.sliderSlice.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.sliderSlice.setFocusPolicy(QtCore.Qt.WheelFocus)

        self.sliderSlice.setTracking(True)


        self.setFocusPolicy(QtCore.Qt.NoFocus)

        self.spinSlice = QtGui.QSpinBox(self)
        self.spinSlice.setStyleSheet("color:white;")
        self.spinSlice.setButtonSymbols(QtGui.QAbstractSpinBox.NoButtons)
        self.spinSlice.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.spinSlice.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 0)


        self.sliderSlice.valueChanged.connect(self.spinSlice.setValue)
        self.spinSlice.valueChanged.connect(self.sliderSlice.setValue)


        self.setStyleSheet("""
        background-color:black;
        color:white;
        """)

        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.checkSlice)

        hbox.addWidget(self.sliderSlice)
        hbox.addWidget(self.spinSlice)

        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(self.glSliceWidget)
        vbox.addLayout(hbox)


        vbox.setSpacing(1)
        hbox.setSpacing(11)

        self.setLayout(vbox)

        self.glSliceWidget._dataModelChanged.connect(self.dataModelChanged)

        self.setTransform(TransformModel())
        self.sliderSlice.setFocus()


        # self.refresh()


    def setTransform(self, transform):
        self.transform = transform
        self.glSliceWidget.setTransform(transform)
        self.checkSlice.stateChanged.connect(self.transform.setSliceDim)
        self.transform._sliceDimChanged.connect(self.update)


    def update(self):
        if self.glSliceWidget.dataModel:
            dim = self.glSliceWidget.dataModel.size()[1:][::-1][self.transform.sliceDim]

            self.sliderSlice.setRange(0,dim-1)
            self.sliderSlice.setValue(self.transform.slicePos)
            self.spinSlice.setRange(0,dim-1)


    def dataModelChanged(self):
        dataModel = self.glSliceWidget.dataModel
        dataModel._dataSourceChanged.connect(self.dataSourceChanged)
        self.glSliceWidget.transform._slicePosChanged.connect(self.sliderSlice.setValue)
        self.dataSourceChanged()

    def dataSourceChanged(self):
        self.sliderSlice.valueChanged.connect(self.glSliceWidget.transform.setSlicePos)
        self.update()

    def setModel(self,dataModel):
        self.glSliceWidget.setModel(dataModel)

    def wheelEvent(self, event):
        self.sliderSlice.wheelEvent(event)


if __name__ == '__main__':
    from spimagine import DemoData, NumpyData

    app = QtGui.QApplication(sys.argv)

    # win = GLSliceWidget(size=QtCore.QSize(500,500))

    win = SliceWidget(size=QtCore.QSize(500,500))


    d = random.uniform(0,1,10*20*30).reshape(10,20,30)

    win.setModel(DataModel(NumpyData(d)))
    # win.setModel(DataModel(DemoData()))

    # win.transform.setBox()
    # win.transform.setPerspective(True)

    win.show()

    win.raise_()

    sys.exit(app.exec_())




import numpy as np

import os

from PyQt4 import Qt, QtCore, QtGui

import logging

logger = logging.getLogger(__name__)




class FloatSlider(QtGui.QSlider):
    floatValueChanged = QtCore.pyqtSignal(float)
    def __init__(self,*args):
        super(FloatSlider,self).__init__(*args)
        self.setRange(0,1.,100)
        self.valueChanged.connect(self.onChanged)

    def setRange(self,minVal,maxVal,steps=100):
        assert(minVal < maxVal)
        super(FloatSlider,self).setRange(0,steps)
        self.minVal = minVal
        self.maxVal = maxVal
        self.steps = steps

    def _from_float(self,x):
        ind = int(self.steps*(x-self.minVal)/(self.maxVal-self.minVal))
        ind = max(0,min(self.steps,ind))
        logger.debug("floatslider:  index from float %s = %s"%(x,ind))
        return ind

    def _from_int(self,n):
        return self.minVal+1.*(self.maxVal-self.minVal)*n/self.steps

    def setValue(self,val):
        logger.debug("floatslider: setValue to : %s"%val)
        self.floatValue = val
        super(FloatSlider,self).setValue(self._from_float(val))

    def value(self):
        return self.floatValue
        # return self._from_int(super(FloatSlider,self).value())

    def onChanged(self,ind):
        self.floatValue = self._from_int(ind)

        self.floatValueChanged.emit(self.floatValue)

        # self.floatValueChanged.emit(self._from_int(ind))



class MainWindow(QtGui.QWidget):

    def __init__(self):
        super(MainWindow,self).__init__()

        self.slide = FloatSlider(QtCore.Qt.Horizontal)
        self.slide.setRange(-12.,10.,400)
        self.slide.valueChanged.connect(self.onSlide)
        self.slide.floatValueChanged.connect(self.onSlideFloat)


        self.slide.setValue(7.)
        self.setWindowTitle("Key Frame View")


        self.edit = QtGui.QLineEdit("")
        self.edit.setValidator(QtGui.QDoubleValidator())
        self.edit.returnPressed.connect(lambda: self.slide.setValue(float(self.edit.text())))


        self.slide.floatValueChanged.connect(lambda x: self.edit.setText(str(x)))



        self.resize(500,200)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.slide)
        hbox.addWidget(self.edit)

        self.setLayout(hbox)


    def onSlide(self,val):
        print "int:\t",val

    def onSlideFloat(self,val):
        print "float:\t",val


if __name__ == '__main__':

    import sys

    app = QtGui.QApplication(sys.argv)

    win = MainWindow()

    win.show()
    win.raise_()

    app.exec_()



import sys
import numpy as np
import os

from PyQt4 import QtCore, QtGui

from collections import OrderedDict


import spimagine

from spimagine.gui.mainwidget import MainWidget


from spimagine.models.data_model import DataModel, SpimData, TiffData, TiffFolderData,GenericData, EmptyData, DemoData, NumpyData

_MAIN_APP = None

#FIXME app issue

import logging
logger = logging.getLogger(__name__)

def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)



def getCurrentApp():
    app = QtGui.QApplication.instance()

    if not app:
        app = QtGui.QApplication(sys.argv)

    if not hasattr(app,"volfigs"):
        app.volfigs = OrderedDict()

    global _MAIN_APP
    _MAIN_APP = app
    return _MAIN_APP



def volfig(num=None, raise_window = True):
    """return window"""

    logger.debug("volfig")


    app = getCurrentApp()
    app.setWindowIcon(QtGui.QIcon(absPath('images/spimagine.png')))
    
    #filter the dict
    app.volfigs =  OrderedDict((n,w) for n,w in app.volfigs.iteritems() if w.isVisible())


    if not num:
        if len(app.volfigs.keys())==0:
            num = 1
        else:
            num = max(app.volfigs.iterkeys())+1

    if app.volfigs.has_key(num):
        window = app.volfigs[num]
        app.volfigs.pop(num)
    else:
        window = MainWidget()
        window.show()
    #make num the last window
    app.volfigs[num] = window

    if raise_window:
        window.raise_()

    return window


def volshow(data, autoscale = True,
            stackUnits = [1.,1.,1.],
            blocking = False,
            cmap = None,
            raise_window = True):
    """
    class to visualize 3d/4d data

    data can be

    - a 3d/4d numpy array of dimensions (z,y,x) or (t,z,y,x)

      e.g.


volshow( randint(0,10,(10, 20,30,40) )


    - an instance of a class derived from the abstract bass class GenericData

      e.g.

from spimagine.data_model import GenericData

class myData(GenericData):
    def __getitem__(self,i):
        return (100*i+3)*ones((100,100,100)
    def size(self):
        return (4,100,100,100)

volshow(myData())

        or
from spimagine.data_model import DataModel

volshow(DataModel(dataContainer=myData(), prefetchSize= 5)



    returns window.glWidget if not in blocking mode


    available colormaps: cmap = ["coolwarm","jet","hot","grays"]
    if cmap = None, then the default one is used

    """

    logger.debug("volshow")

    logger.debug("volshow: getCurrentApp")

    app = getCurrentApp()



    from time import time

    t = time()

    # check whether there are already open windows, if not create one
    try:
        num,window = [(n,w) for n,w in app.volfigs.iteritems()][-1]
    except:
        num = 1

    window = volfig(num, raise_window = raise_window)

    logger.debug("volfig: %s s "%(time()-t))
    t = time()

    # if isinstance(data,GenericData):
    if hasattr(data,"stackUnits"):
        m = DataModel(data)
    elif isinstance(data,DataModel):
        m = data
    else:
        if not isinstance(data,np.ndarray):
            data = np.array(data)
        if autoscale:
            ma,mi = np.amax(data), np.amin(data)
            if ma==mi:
                ma +=1.
            data = 1000.*(data-mi)/(ma-mi)

        m = DataModel(NumpyData(data.astype(np.float32,copy=False)))


    logger.debug("create model: %s s "%( time()-t))
    t = time()

    window.setModel(m)

    if cmap is None or not spimagine.config.__COLORMAPDICT__.has_key(cmap):
        cmap = spimagine.config.__DEFAULTCOLORMAP__


    window.glWidget.set_colormap(cmap)



    logger.debug("set model: %s s"%( time()-t));
    t = time()

    window.glWidget.transform.setStackUnits(*stackUnits)

    if blocking:
        getCurrentApp().exec_()
    else:
        return window


def qt_exec():
    getCurrentApp().exec_()


class TimeData(GenericData):
    def __init__(self,func, dshape):
        """ func(i) returns the volume
        dshape is [Nt,Nz,Nx,Ny]
        """
        self.func = func
        self.dshape = dshape

        GenericData.__init__(self)

    def __getitem__(self,i):
        return self.func(i)

    def size(self):
        return self.dshape


if __name__ == '__main__':

    volshow(DemoData(),blocking = True)




import sys
from PyQt4 import QtCore
from PyQt4 import QtGui
from PyQt4 import QtOpenGL
from OpenGL import GLU
from OpenGL import GLUT
from OpenGL.GL import *
from numpy import *
from transform_matrices import *

class GLWidget(QtOpenGL.QGLWidget):
    def __init__(self, parent=None):
        self.parent = parent
        super(GLWidget,self).__init__(parent)

    def initializeGL(self):
        self.qglClearColor(QtGui.QColor(0, 0,  0))

        # glEnable(GL_DEPTH_TEST)
        glEnable(GL_TEXTURE_2D)
        glEnable(GL_BLEND)
        # glEnable(GL_DEPTH_TEST)
        glEnable( GL_LINE_SMOOTH )
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        self.texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        glPixelStorei(GL_UNPACK_ALIGNMENT,1)
        glTexParameterf (GL_TEXTURE_2D,
                            GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameterf (GL_TEXTURE_2D,
                            GL_TEXTURE_MAG_FILTER, GL_LINEAR)

        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameterf (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)



    def resizeGL(self, width, height):
        if height == 0: height = 1

        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        aspect = width / float(height)

        GLU.gluPerspective(45.0, aspect, 1.0, 100.0)
        glMatrixMode(GL_MODELVIEW)

    def paintGL(self):
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()

        Nx, Ny = 100,100
        self.output = linspace(0,1.,Nx*Ny)

        glBindTexture(GL_TEXTURE_2D,self.texture)
        # glTexImage2D(GL_TEXTURE_2D, 0, 1, Ny, Nx,
        #                  0, GL_LUMINANCE, GL_UNSIGNED_BYTE, self.output.astype(uint8))

        glTexImage2D(GL_TEXTURE_2D, 0, 1, Ny, Nx,
                         0, GL_RED, GL_FLOAT, self.output.astype(float32))

        w,h = 1,1
        glBegin (GL_QUADS);
        glTexCoord2f (0, 0);
        glVertex2f (-w, -h);
        glTexCoord2f (1, 0);
        glVertex2f (w, -h);
        glTexCoord2f (1, 1);
        glVertex2f (w,h);
        glTexCoord2f (0, 1);
        glVertex2f (-w, h);
        glEnd();

        # glOrtho(-2,2,-2,2,-10,10)

        # print glGetFloatv(GL_PROJECTION_MATRIX).T
        # print projMatOrtho(-2.,2.,-2.,2.,-10,10)
        # glMatrixMode(GL_MODELVIEW)
        # glLoadIdentity()



        # glLineWidth(1)
        # glColor(1.,1.,1.,1.)
        # GLUT.glutWireCube(2)


    def onTimer(self):
        self.updateGL()

class MainWindow(QtGui.QMainWindow):

    def __init__(self):
        super(MainWindow,self).__init__()

        self.resize(300, 300)
        self.setWindowTitle('GL Cube Test')

        self.initActions()
        self.initMenus()


        glWidget = GLWidget(self)
        self.setCentralWidget(glWidget)

        timer = QtCore.QTimer(self)
        timer.setInterval(20)
        QtCore.QObject.connect(timer, QtCore.SIGNAL('timeout()'), glWidget.onTimer)
        timer.start()


    def initActions(self):
        self.exitAction = QtGui.QAction('Quit', self)
        self.exitAction.setShortcut('Ctrl+Q')
        self.exitAction.setStatusTip('Exit application')
        self.connect(self.exitAction, QtCore.SIGNAL('triggered()'), self.close)

    def initMenus(self):
        menuBar = self.menuBar()
        fileMenu = menuBar.addMenu('&File')
        fileMenu.addAction(self.exitAction)
        # this has to be repeated in MAC OSX for some magic reason
        fileMenu = menuBar.addMenu('&File')

    def close(self):
        QtGui.qApp.quit()


if __name__ == '__main__':

    app = QtGui.QApplication(sys.argv)

    win = MainWindow()
    win.show()
    win.raise_()

    sys.exit(app.exec_())




import logging
logger = logging.getLogger(__name__)


import sys
import os
from PyQt4 import QtCore
from PyQt4 import QtGui
from PyQt4 import QtOpenGL

import numpy as np

from spimagine.gui.floatslider import FloatSlider

from spimagine.gui.gui_utils import createImageCheckbox,createStandardButton

import spimagine


def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


checkBoxStyleStr = """
    QCheckBox::indicator:checked {
    background:black;
    border-image: url(%s);}
    QCheckBox::indicator:unchecked {
    background:black;
    border-image: url(%s);}
    """

# def createImgCheckBox(parentfName_active,fName_inactive):
#     checkBox = QtGui.QCheckBox()
#     checkBox.setStyleSheet(
#             checkBoxStyleStr%(absPath(fName_active),absPath(fName_inactive)))
#     return checkBox

# def createImgPushButton(parent,fName, tooltip = ""):
#     but = QtGui.QPushButton("",parent)
#     but.setStyleSheet("background-color: black;color:black")
#     but.setIcon(QtGui.QIcon(absPath(fname)))
#     but.setIconSize(QtCore.QSize(24,24))
#     but.clicked.connect(self.folderSelect)


class MainSettingsPanel(QtGui.QWidget):
    _playIntervalChanged = QtCore.pyqtSignal(int)
    _substepsChanged = QtCore.pyqtSignal(int)

    _dirNameChanged =  QtCore.pyqtSignal(str)
    _frameNumberChanged = QtCore.pyqtSignal(int)
    
    def __init__(self):
        super(QtGui.QWidget,self).__init__()

        self.resize(50, 300)
        self.initUI()


    def initUI(self):


        vbox = QtGui.QVBoxLayout()


        vbox.addWidget(QtGui.QLabel("Render",alignment = QtCore.Qt.AlignCenter))

        gridBox = QtGui.QGridLayout()

        gridBox.addWidget(QtGui.QLabel("loop bounce:\t"),0,0)
        self.checkLoopBounce = QtGui.QCheckBox()
        gridBox.addWidget(self.checkLoopBounce,0,1)


        gridBox.addWidget(QtGui.QLabel("play interval (ms):\t"))
        self.playInterval = QtGui.QLineEdit("50")
        self.playInterval.setValidator(QtGui.QIntValidator(bottom=10))
        self.playInterval.returnPressed.connect(self.playIntervalChanged)
        gridBox.addWidget(self.playInterval)

        gridBox.addWidget(QtGui.QLabel("subrender steps:\t"))
        self.editSubsteps = QtGui.QLineEdit("1")
        self.editSubsteps.setValidator(QtGui.QIntValidator(bottom=1))
        self.editSubsteps.returnPressed.connect(self.substepsChanged)
        gridBox.addWidget(self.editSubsteps)

        gridBox.addWidget(QtGui.QLabel("Egg3D:\t"))
        self.checkEgg = createImageCheckbox(self, absPath("images/egg.png"),
                                            absPath("images/egg_inactive.png"),
                                            tooltip="toggle egg control")

        gridBox.addWidget(self.checkEgg)

        vbox.addLayout(gridBox)

        line =  QtGui.QFrame()
        line.setFrameShape(QtGui.QFrame.HLine)

        vbox.addWidget(line)

        vbox.addWidget(QtGui.QLabel("General",
                                    alignment = QtCore.Qt.AlignCenter))

        renderFolder = QtGui.QLineEdit("./")
        hbox = QtGui.QHBoxLayout()

        hbox.addWidget(QtGui.QLabel("output folder: ",
                                    alignment = QtCore.Qt.AlignCenter))

        hbox.addWidget(renderFolder)

        folderButton = createStandardButton(self,
                        absPath("images/icon_folder.png"),
                        tooltip = "folder to render to")
        
        folderButton.clicked.connect(self.folderSelect)

        hbox.addWidget(folderButton)

        renderFolder.returnPressed.connect(lambda: self.setDirName(renderFolder.text()))
        self._dirNameChanged.connect(renderFolder.setText)

        self.setDirName("./")

        vbox.addLayout(hbox)

        
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(QtGui.QLabel("number frames:\t"))
        frameEdit = QtGui.QLineEdit("100")
        frameEdit.setValidator(QtGui.QIntValidator(bottom=1))
        frameEdit.returnPressed.connect(lambda: self._frameNumberChanged.emit(int(frameEdit.text())))
        hbox.addWidget(frameEdit)

        vbox.addLayout(hbox)

        self.setStyleSheet("""
        QFrame,QLabel,QLineEdit {
        color: white;
        }
        """)
        vbox.addStretch()

        self.setLayout(vbox)



    def setDirName(self,dirName):
        logger.debug("setDirName: %s"%dirName)
        self.dirName = dirName
        self._dirNameChanged.emit(dirName)


    def folderSelect(self,event):
        dirName= QtGui.QFileDialog.getExistingDirectory(self, 'select output folder',
                self.dirName)
        if dirName:
            self.setDirName(dirName)

    def onButtonColor(self):
        col = QtGui.QColorDialog.getColor()

        if col.isValid():
            color = 1./255*np.array(col.getRgb()[:3])
            self._rgbColorChanged.emit(*color)

    def setStackUnits(self,px,py,pz):
        for e,p in zip(self.stackEdits,[px,py,pz]):
            e.setText(str(p))

    def setBounds(self,x1,x2,y1,y2,z1,z2):
        for x,s in zip([x1,x2,y1,y2,z1,z2],self.sliderBounds):
            flag = s.blockSignals(True)
            s.setValue(x*100)
            s.blockSignals(flag)


    def boundsChanged(self):
        bounds = [s.value()/100. for s in self.sliderBounds]
        self._boundsChanged.emit(*bounds)


    def alphaPowChanged(self):
        alphaPow = 100.*(self.sliderAlphaPow.value()/100.)**3
        self._alphaPowChanged.emit(alphaPow)

    def stackUnitsChanged(self):
        try:
            stackUnits = [float(e.text()) for e in self.stackEdits]
            self._stackUnitsChanged.emit(*stackUnits)
        except Exception as e:
            print "couldnt parse text"
            print e

    def playIntervalChanged(self):
        self._playIntervalChanged.emit(int(self.playInterval.text()))

    def substepsChanged(self):
        print "changed substeps to ", int(self.editSubsteps.text())
        self._substepsChanged.emit(int(self.editSubsteps.text()))
        
class MainWindow(QtGui.QMainWindow):

    def __init__(self, ):
        super(QtGui.QMainWindow,self).__init__()

        self.resize(500, 300)
        self.setWindowTitle('Test')

        self.settings = MainSettingsPanel()
        self.setCentralWidget(self.settings)
        self.setStyleSheet("background-color:black;")

    def close(self):
        QtGui.qApp.quit()


if __name__ == '__main__':

    app = QtGui.QApplication(sys.argv)

    win = MainWindow()
    win.show()
    win.raise_()

    sys.exit(app.exec_())



import logging
logger = logging.getLogger(__name__)

import os

from myconfigparser import MyConfigParser        
from loadcolormaps import loadcolormaps



__CONFIGFILE__ = os.path.expanduser("~/.spimagine")
            
__spimagine_config_parser = MyConfigParser(__CONFIGFILE__)

__OPENCLDEVICE__ = int(__spimagine_config_parser.get("opencldevice",0))
__DEFAULTCOLORMAP__ = __spimagine_config_parser.get("colormap","viridis")


__DEFAULTWIDTH__ = int(__spimagine_config_parser.get("width",800))
__DEFAULTMAXSTEPS__ = int(__spimagine_config_parser.get("max_steps",200))

__COLORMAPDICT__ = loadcolormaps()


def setOpenCLDevice(num):
    global __OPENCLDEVICE__
    __OPENCLDEVICE__ = num


#this should fix an annoying file url drag drop bug in mac yosemite
import platform

__SYSTEM_DARWIN__ = False

if platform.system() =="Darwin":
    __SYSTEM_DARWIN__ = True
    def _parseFileNameFix(fpath):
        from subprocess import check_output
        path  = check_output(["osascript","-e","get posix path of posix file \"file://%s\" -- kthxbai"%fpath])
        print path[:-1]
        return path[:-1]


if __name__ == '__main__':
    pass



#!/usr/bin/env python

"""
the (Qt) data models for usage in the gui frame

generic containers are defined for BScope Spim Data (SpimData)
and Tiff files (TiffData).
Extend it if you want to and change the DataLoadModel.chooseContainer to
accept it via dropg

author: Martin Weigert
email: mweigert@mpi-cbg.de
"""

import logging
logger = logging.getLogger(__name__)


import os
import numpy as np
import time

from PyQt4 import QtCore



import re
import glob


# import h5py


from collections import defaultdict

import spimagine.utils.imgutils as imgutils

def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)


############################################################################
"""
The next classes define simple 4d Data Structures that implement the interface
given by GenericData
"""

class GenericData():
    """abstract base class for 4d data

    if you wanna sublass it, just overwrite self.size() and self.__getitem__()

    """
    dataFileError = Exception("not a valid file")
    def __init__(self, name = ""):
        self.stackSize = None
        self.stackUnits = None
        self.name = name

    # def setStackSize(self, stackSize):
    #     self.stackSize  = list(stackSize)

    def sizeT(self):
        return self.size()[0]

    def size(self):
        return self.stackSize

    def __getitem__(self,pos):
        return None
    # #this should be override by every derived class
    # def _get_single_item(self,i):
    #     return None
    #
    # def __getslice__(self, i, j):
    #     return self.__getitem__(slice(i, j))
    #
    # def __getitem__(self,i):
    #     if isinstance(key, slice):
    #         return [self.list[i] for i in xrange(key.start, key.stop, key.step)]
    #     else:
    #         self._get_single_item(i)


class SpimData(GenericData):
    """data class for spim data saved in folder fName
    fname/
    |-- metadata.txt
    |-- data/
       |--data.bin
       |--index.txt
    """
    def __init__(self,fName = ""):
        GenericData.__init__(self, fName)
        self.load(fName)

    def load(self,fName):
        if fName:
            try:
                self.stackSize = imgutils.parseIndexFile(os.path.join(fName,"data/index.txt"))
                self.stackUnits = imgutils.parseMetaFile(os.path.join(fName,"metadata.txt"))
                self.fName = fName
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as SpimData"%fName)

            try:
                # try to figure out the dimension of the dark frame stack
                darkSizeZ = os.path.getsize(os.path.join(self.fName,"data/darkstack.bin"))/2/self.stackSize[2]/self.stackSize[3]
                with open(os.path.join(self.fName,"data/darkstack.bin"),"rb") as f:
                    self.darkStack = np.fromfile(f,dtype="<u2").reshape([darkSizeZ,self.stackSize[2],self.stackSize[3]])

            except Exception as e:
                logger.warning("couldn't find darkstack (%s)",e)


    def __getitem__(self,pos):
        if self.stackSize and self.fName:
            if pos<0 or pos>=self.stackSize[0]:
                raise IndexError("0 <= pos <= %i, but pos = %i"%(self.stackSize[0]-1,pos))


            pos = max(0,min(pos,self.stackSize[0]-1))
            voxels = np.prod(self.stackSize[1:])
            # use int64 for bigger files
            offset = np.int64(2)*pos*voxels

            with open(os.path.join(self.fName,"data/data.bin"),"rb") as f:
                f.seek(offset)
                return np.fromfile(f,dtype="<u2",
                count=voxels).reshape(self.stackSize[1:])
        else:
            return None




class Img2dData(GenericData):
    """2d image data"""
    def __init__(self,fName = ""):
        GenericData.__init__(self, fName)
        self.load(fName)

    def load(self,fName, stackUnits = [1.,1.,1.]):
        if fName:
            try:
                self.img  = np.array([imgutils.openImageFile(fName)])
                self.stackSize = (1,) + self.img.shape
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as Img2dData"%fName)
                return

            self.stackUnits = stackUnits
            self.fName = fName


    def __getitem__(self,pos):
        if self.stackSize and self.fName:
            return self.img
        else:
            return None


class TiffData(GenericData):
    """3d tiff data"""
    def __init__(self,fName = ""):
        GenericData.__init__(self, fName)
        self.load(fName)

    def load(self,fName, stackUnits = [1.,1.,1.]):
        if fName:
            try:
                data = np.squeeze(imgutils.read3dTiff(fName))

                if not data.ndim in [3,4]:
                    raise ValueError("in file %s: dada.ndim = %s (not 3 or 4)"%(fName,data.ndim))

                if data.ndim == 3:
                    self.stackSize = (1,)+ data.shape
                else:
                    self.stackSize = data.shape
                self.data = data
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as TiffData"%fName)
                return

            self.stackUnits = stackUnits
            self.fName = fName


    def __getitem__(self,pos):
        if self.stackSize and self.fName:
            if self.data.ndim == 3:
                return self.data
            else:
                return self.data[pos]


        else:
            return None



class TiffFolderData(GenericData):
    """3d tiff data inside a folder"""
    def __init__(self,fName = ""):
        GenericData.__init__(self, fName)
        self.fNames = []
        self.fName = ""
        self.load(fName)

    def load(self,fName, stackUnits = [1.,1.,1.]):
        if fName:
            self.fNames = glob.glob(os.path.join(fName,"*.tif"))
            if len(self.fNames) == 0:
                raise Exception("folder %s seems to be empty"%fName)
            else:
                self.fNames.sort()
            
            try:
                _tmp = imgutils.read3dTiff(self.fNames[0])
                self.stackSize = (len(self.fNames),)+ _tmp.shape
            except Exception as e:
                print e
                self.fName = ""
                raise Exception("couldnt open %s as TiffData"%self.fNames[0])
                return

            self.stackUnits = stackUnits
            self.fName = fName


    def __getitem__(self,pos):
        if len(self.fNames)>0 and pos<len(self.fNames):
            return imgutils.read3dTiff(self.fNames[pos])

class TiffMultipleFiles(GenericData):
    """3d tiff data inside a folder"""
    def __init__(self,fName = []):
        GenericData.__init__(self, fName)
        self.fNames = fName
        self.load(fName)

    def load(self,fNames, stackUnits = [1.,1.,1.]):
        if fNames:
            if len(self.fNames) == 0:
                raise Exception("filelist %s seems to be empty"%fName)

            try:
                _tmp = imgutils.read3dTiff(self.fNames[0])
                self.stackSize = (len(self.fNames),)+ _tmp.shape
            except Exception as e:
                print e
                raise Exception("couldnt open %s as TiffData"%self.fNames[0])
                return

            self.stackUnits = stackUnits


    def __getitem__(self,pos):
        if len(self.fNames)>0 and pos<len(self.fNames):
            return imgutils.read3dTiff(self.fNames[pos])

class NumpyData(GenericData):

    def __init__(self, data, stackUnits = [1.,1.,1.]):
        GenericData.__init__(self,"NumpyData")

        if len(data.shape)==3:
            self.stackSize = (1,) + data.shape
            self.data = data.copy().reshape(self.stackSize)
        elif len(data.shape)==4:
            self.stackSize = data.shape
            self.data = data.copy()
        else:
            raise TypeError("data should be 3 or 4 dimensional! shape = %s" %str(data.shape))


        self.stackUnits = stackUnits

    def __getitem__(self,pos):
        return self.data[pos,...]


class DemoData(GenericData):
    def __init__(self, N = None):
        GenericData.__init__(self,"DemoData")
        self.load(N)

    def load(self,N = None):
        if N==None:
            logger.debug("loading precomputed demodata")
            self.data = imgutils.read3dTiff(absPath("../data/mpi_logo_80.tif")).astype(np.float32)
            N = 80
            self.stackSize = (10,N,N,N)
            self.fName = ""
            self.nT = 10
            self.stackUnits = (1,1,1)

        else:
            self.stackSize = (1,N,N,N)
            self.fName = ""
            self.nT = N
            self.stackUnits = (1,1,1)
            x = np.linspace(-1,1,N)
            Z,Y,X = np.meshgrid(x,x,x , indexing = "ij")
            R = np.sqrt(X**2+Y**2+Z**2)
            R2 = np.sqrt((X-.4)**2+(Y+.2)**2+Z**2)
            phi = np.arctan2(Z,Y)
            theta = np.arctan2(X,np.sqrt(Y**2+Z**2))
            u = np.exp(-500*(R-1.)**2)*np.sum(np.exp(-150*(-theta-t+.1*(t-np.pi/2.)*
                np.exp(-np.sin(2*(phi+np.pi/2.))))**2)
                for t in np.linspace(-np.pi/2.,np.pi/2.,10))*(1+Z)

            u2 = np.exp(-7*R2**2)
            self.data = (10000*(u + 2*u2)).astype(np.float32)


    def sizeT(self):
        return self.nT

    def __getitem__(self,pos):
        return (self.data*np.exp(-.3*pos)).astype(np.float32)


class EmptyData(GenericData):
    def __init__(self):
        GenericData.__init__(self,"EmptyData")
        self.stackSize = (1,1,1,1)
        self.fName = ""
        self.nT = 1
        self.stackUnits = (1,1,1)
        self.data = np.zeros((1,1,1)).astype(np.uint16)

    def sizeT(self):
        return self.nT

    def __getitem__(self,pos):
        return self.data


# class HDF5Data(GenericData):
#     """loads hdf5 data files
#     """

#     def __init__(self,fName = None, key = None ):
#         GenericData.__init__(self, fName)
#         self.load(fName, key)

#     def load(self,fName, key = None, stackUnits = [1.,1.,1.]):
#         if fName:
#             with h5py.File(fName,"r") as f:
#                 if len(f.keys())==0 :
#                     raise KeyError("no valid key found in file %s"%fName)
#                 if key is None:
#                     key = f.keys()[0]
#                 self.data = np.asarray(f[key][:]).copy()
#                 self.stackSize = (1,)+ self.data.shape
#                 self.stackUnits = stackUnits
#                 self.fName = fName
        
#     def __getitem__(self,pos):
#         return self.data

class CZIData(GenericData):
    """loads czi data files
    """

    def __init__(self,fName = None):
        GenericData.__init__(self, fName)
        self.load(fName)

    def load(self,fName, stackUnits = [1.,1.,1.]):

        if fName:
            try:
                data = np.squeeze(imgutils.readCziFile(fName))

                if not data.ndim in [3,4]:
                    raise ValueError("in file %s: dada.ndim = %s (not 3 or 4)"%(fName,data.ndim))

                if data.ndim == 3:
                    self.stackSize = (1,)+ data.shape
                else:
                    self.stackSize = data.shape
                self.data = data
                self.stackUnits = stackUnits
                self.fName = fName
            except Exception as e:
                print e
        
    def __getitem__(self,pos):
        if self.data.ndim == 3:
            return self.data
        else:
            return self.data[pos]



############################################################################
"""
this defines the qt enabled data models based on the GenericData structure

each dataModel starts a prefetching thread, that loads next timepoints in
the background
"""


class DataLoadThread(QtCore.QThread):
    """the prefetching thread for each data model"""
    def __init__(self, _rwLock, nset = set(), data = None,dataContainer = None):
        QtCore.QThread.__init__(self)
        self._rwLock = _rwLock
        if nset and data and dataContainer:
            self.load(nset, data, dataContainer)


    def load(self, nset, data, dataContainer):
        self.nset = nset
        self.data = data
        self.dataContainer = dataContainer


    def run(self):
        import time

        self.stopped = False

        while not self.stopped:
            self._rwLock.lockForWrite()

            kset = set(self.data.keys())
            dkset = kset.difference(set(self.nset))
            dnset = set(self.nset).difference(kset)

            for k in dkset:
                del(self.data[k])

            self._rwLock.unlock()
            
            if dnset:
                logger.debug("preloading %s", list(dnset))
                for k in dnset:
                    newdata = self.dataContainer[k]
                    self._rwLock.lockForWrite()
                    self.data[k] = newdata
                    self._rwLock.unlock()
                    logger.debug("preload: %s",k)
                    try:
                        time.sleep(.0001)
                    except Exception as e:
                        print e

            # print "load thead dict length: ", len(self.data.keys())
            
            try:
                time.sleep(.0001)
            except Exception as e:
                print e


class DataModel(QtCore.QObject):
    """the data model
    emits signals when source/time position has changed
    """
    _dataSourceChanged = QtCore.pyqtSignal()
    _dataPosChanged = QtCore.pyqtSignal(int)

    _rwLock = QtCore.QReadWriteLock()

    def __init__(self, dataContainer = None, prefetchSize = 0):
        assert prefetchSize>=0
        
        super(DataModel,self).__init__()
        self.dataLoadThread = DataLoadThread(self._rwLock)
        self._dataSourceChanged.connect(self.dataSourceChanged)
        self._dataPosChanged.connect(self.dataPosChanged)
        if dataContainer:
            self.setContainer(dataContainer, prefetchSize)

    @classmethod
    def fromPath(self,fName, prefetchSize = 0):
        d = DataModel()
        d.loadFromPath(fName,prefetchSize)
        return d

    def setContainer(self,dataContainer = None, prefetchSize = 0):
        self.dataContainer = dataContainer
        self.prefetchSize = prefetchSize
        self.nset = [0]
        self.data = defaultdict(lambda: None)

        if self.dataContainer:
            self.stopDataLoadThread()
            self.dataLoadThread.load(self.nset,self.data, self.dataContainer)
            self.dataLoadThread.start(priority=QtCore.QThread.LowPriority)
            self._dataSourceChanged.emit()
            self.setPos(0)


    def __repr__(self):
        return "DataModel: %s \t %s"%(self.dataContainer.name,self.size())

    def dataSourceChanged(self):
        logger.debug("data source changed:\n%s",self)

    def dataPosChanged(self, pos):
        logger.debug("data position changed to %i",pos)



    def stopDataLoadThread(self):
        self.dataLoadThread.stopped = True

    def prefetch(self,pos):
        self._rwLock.lockForWrite()
        self.nset[:] = self.neighborhood(pos)
        self._rwLock.unlock()

    def sizeT(self):
        if self.dataContainer:
            return self.dataContainer.sizeT()

    def size(self):
        if self.dataContainer:
            return self.dataContainer.size()

    def name(self):
        if self.dataContainer:
            return self.dataContainer.name

    def stackUnits(self):
        if self.dataContainer:
            return self.dataContainer.stackUnits

    def setPos(self,pos):
        if pos<0 or pos>=self.sizeT():
            raise IndexError("setPos(pos): %i outside of [0,%i]!"%(pos,self.sizeT()-1))
            return

        if not hasattr(self,"pos") or self.pos != pos:
            self.pos = pos
            self._dataPosChanged.emit(pos)
            self.prefetch(self.pos)


    def __getitem__(self,pos):
        # self._rwLock.lockForRead()
        if not hasattr(self,"data"):
            print "something is wrong in datamodel as its lacking a 'data' atttribute!"
            return None


        # switching of the prefetched version for now...
        # as for some instances there seems to be a race condition still
        
        if not self.data.has_key(pos):
            newdata = self.dataContainer[pos]
            self._rwLock.lockForWrite()
            self.data[pos] = newdata
            self._rwLock.unlock()

        self.prefetch(pos)

        return self.data[pos]

        # return self.dataContainer[pos]



    def neighborhood(self,pos):
        # FIXME mod stackSize!
        return np.arange(pos,pos+self.prefetchSize+1)%self.sizeT()

    def loadFromPath(self,fName, prefetchSize = 0):
        
        if  isinstance(fName,(tuple,list)):
            self.setContainer(TiffMultipleFiles(fName),prefetchSize)

        elif re.match(".*\.tif",fName):
            self.setContainer(TiffData(fName),prefetchSize = 0)
        elif re.match(".*\.(png|jpg|bmp)",fName):
            self.setContainer(Img2dData(fName),prefetchSize = 0)
        # elif re.match(".*\.h5",fName):
        #     self.setContainer(HDF5Data(fName),prefetchSize = 0)
        elif re.match(".*\.czi",fName):
            self.setContainer(CZIData(fName),prefetchSize = 0)
        elif os.path.isdir(fName):
            if os.path.exists(os.path.join(fName,"metadata.txt")):
                self.setContainer(SpimData(fName),prefetchSize)
            else:
                self.setContainer(TiffFolderData(fName),prefetchSize = prefetchSize)            





def test_spimdata():
    d = SpimData("/Users/mweigert/Data/HisGFP")

    m = DataModel(d)
    print m
    for pos in range(m.sizeT()):
        print pos
        print np.mean(m[pos])


def test_tiffdata():
    d = TiffData("/Users/mweigert/Data/droso_test.tif")

    m = DataModel(d)
    print m
    for pos in range(m.sizeT()):
        print pos
        print np.mean(m[pos])


def test_numpydata():
    d = NumpyData(np.ones((10,100,100,100)))


    m = DataModel(d)

    print m
    for pos in range(m.sizeT()):
        print pos
        print np.mean(m[pos])

def test_frompath():
    m = DataModel.fromPath("/Users/mweigert/Data/HisGFP")
    m = DataModel.fromPath("/Users/mweigert/Data/droso_test.tif")


def test_speed():
    import time

    fName  = "/Users/mweigert/Data/Drosophila_full"

    t = []
    d = DataModel.fromPath(fName,1)

    for i in range(100):
        print i

        if i%10==0:
            a = d[i/10]


        time.sleep(.1)
        t.append(time.time())


def test_data_sets():
    fNames = ["test_data/Drosophila_Single",
              "test_data/HisStack_uint16_0000.tif",
              "test_data/HisStack_uint8_0000.tif",
              "test_data/meep.h5",
              "test_data/retina.czi"
          ]

    
    for fName in fNames:
        try:
            d = DataModel.fromPath(fName)
            print fName, d[0].shape
        except Exception as e:
            print e
            print "ERROR    could not open %s"%fName
            

    
        

if __name__ == '__main__':

    # test_data_sets()
    # test_spimdata()

    # test_tiffdata()
    # test_numpydata()

    # test_speed()

    # test_frompath()


    d = DataModel(TiffFolderData("/home/martin/Data_new/Tomancak_Droso/volumes"), prefetchSize= 0)

    for i in np.random.randint(0,d.sizeT(),100):
        print i
        a = d[i]




import os
import sys
import shutil
import numpy as np

from PyQt4 import QtGui, QtCore

from spimagine.gui.mainwidget import MainWidget
from spimagine.models.data_model import DataModel, NumpyData

from gputools import OCLProgram

CACHEDIRS = ["~/.nv/ComputeCache","~/.cache/pyopencl/pyopencl-compiler-cache-v2-py2.7.6.final.0"]

CACHEDIRS = [os.path.expanduser(_C) for _C in CACHEDIRS]

import spimagine

def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)



class MyWidget(MainWidget):
    def __init__(self):
        super(MyWidget,self).__init__()

        self.compileTimer = QtCore.QTimer(self)
        self.compileTimer.setInterval(1000)
        self.compileTimer.timeout.connect(self.on_compile_timer)
        self.compileTimer.start()

    def on_compile_timer(self):
        for c in CACHEDIRS:
            if os.path.exists(c):
                print "removing cache: ", c
                shutil.rmtree(c)

        print "compiling..."


        try:
            dirname = os.path.dirname(spimagine.volumerender.__file__)
            proc = OCLProgram(os.path.join(dirname,"kernels/volume_kernel.cl"),
                                   build_options =
                                      ["-cl-fast-relaxed-math",
                                    "-cl-unsafe-math-optimizations",
                                    "-cl-mad-enable",
                                    "-I %s" %os.path.join(dirname,"kernels/"),
                                    "-D maxSteps=%s"%spimagine.config.__DEFAULTMAXSTEPS__]
                                   )

            self.glWidget.renderer.proc = proc
            self.glWidget.refresh()
            print np.amin(self.glWidget.output),np.amax(self.glWidget.output)



        except Exception as e:
            print e








if __name__ == '__main__':


    x = np.linspace(-1,1,128)
    Z,Y,X = np.meshgrid(x,x,x)
    R1 = np.sqrt((X+.2)**2+(Y+.2)**2+(Z+.2)**2)
    R2 = np.sqrt((X-.2)**2+(Y-.2)**2+(Z-.2)**2)
    d = np.exp(-10*R1**2)+np.exp(-10*R2**2)

    app = QtGui.QApplication(sys.argv)

    win = MyWidget()

    win.setModel(DataModel(NumpyData(d)))

    win.show()

    win.raise_()

    sys.exit(app.exec_())




#!/usr/bin/env python

"""
the main frame used for in spimagine_gui

the data model is member of the frame

author: Martin Weigert
email: mweigert@mpi-cbg.de
"""



import os
import numpy as np
import sys

from PyQt4 import QtCore
from PyQt4 import QtGui



from spimagine.utils.quaternion import Quaternion
from spimagine.gui.glwidget import GLWidget

from spimagine.models.keyframe_model import KeyFrameList, KeyFrame

from spimagine.gui.keyframe_view import KeyFramePanel

from spimagine.gui.mainsettings import MainSettingsPanel

from spimagine.gui.volsettings import VolumeSettingsPanel

from spimagine.models.data_model import DataModel, DemoData, SpimData, TiffData, NumpyData


from spimagine.gui.slice_view import SliceWidget


from spimagine.gui.imageprocessor_view import ImageProcessorListView

import spimagine.utils.egg3d 
from spimagine.models.imageprocessor import *


from spimagine.gui.floatslider import FloatSlider

from spimagine.models.transform_model import TransformModel

from spimagine.gui.gui_utils import  createImageCheckbox,createStandardButton

from spimagine.utils.imgutils import write3dTiff

# from spimagine.jack_plugin import JackPlugin

import logging
logger = logging.getLogger(__name__)


# the default number of data timeslices to prefetch
N_PREFETCH = 10


def absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.debug("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)



class MainWidget(QtGui.QWidget):
    N_SCALE_MIN_EXP = -16
    N_SCALE_MAX_EXP = 17
    N_SCALE_SLIDER = 500

    def __init__(self, parent = None):
        super(QtGui.QWidget,self).__init__(parent)

        self.myparent = parent

        self.isCloseFlag = False

        self.isFullScreen = False
        self.setWindowTitle('SpImagine')

        self.resize(900, 700)


        self.transform = TransformModel()

        self.initActions()

        self.glWidget = GLWidget(self)
        self.glWidget.setTransform(self.transform)

        self.sliceWidget = SliceWidget(self)
        self.sliceWidget.hide()

        # self.jack = JackPlugin(self.transform)
        # self.jack.start()

        
        self.sliceWidget.setTransform(self.transform)

        self.fwdButton = createStandardButton(self,
                fName = absPath("images/icon_forward.png"),
                method = self.forward, width = 18, tooltip="forward")
        self.bwdButton = createStandardButton(self,
                fName = absPath("images/icon_backward.png"),
                method = self.backward, width = 18, tooltip="backward")

        self.startButton = createStandardButton(self, fName = absPath("images/icon_start.png"),
                                                method = self.startPlay, tooltip="play")



        self.centerButton = createStandardButton(self,
                        fName = absPath("images/icon_center.png"),
                        method = self.center, tooltip = "center view")

        self.rotateButton = createStandardButton(self,
                        fName = absPath("images/icon_rotate.png"),
                        method = self.rotate, tooltip = "spin current view")

        self.screenshotButton = createStandardButton(self,
                        fName = absPath("images/icon_camera.png"),
                        method = self.screenShot,
                        tooltip = "save as png")

        self.fileOpenButton = createStandardButton(self,
                        fName = absPath("images/icon_open.png"),
                        method = self.openFile, tooltip = "open file")

        self.fileSaveButton = createStandardButton(self,
                        fName = absPath("images/icon_filesave.png"),
                        method = self.saveFile, tooltip = "save file as tif")
        
        self.checkVolSettings = createImageCheckbox(self,
                                                    absPath("images/icon_volsettings_active.png"),
                                                    absPath("images/icon_volsettings_inactive.png"),
                                                    tooltip="volume settings")

        self.checkSettings = createImageCheckbox(self,
                                                 absPath("images/icon_mainsettings_active.png"),
                                                 absPath("images/icon_mainsettings_inactive.png"),
                                                 tooltip="general settings")


        self.checkKey = createImageCheckbox(self, absPath("images/video.png"), absPath("images/video_inactive.png"), tooltip="keyframe editor")

        self.checkIsoView = createImageCheckbox(self,
                                                absPath("images/icon_method_vol.png"),
                                                absPath("images/icon_method_iso.png"),
                                                tooltip="iso surface")

        self.checkProcView = createImageCheckbox(self,
                                                 absPath("images/icon_process_active.png"),
                                                 absPath("images/icon_process_inactive.png"),
                                                 tooltip="image processors")


        self.checkSliceView = createImageCheckbox(
            self,
            absPath("images/icon_slice.png"),
            absPath("images/icon_slice_inactive.png"), tooltip="slice view")


        self.sliderTime = QtGui.QSlider(QtCore.Qt.Horizontal)
        self.sliderTime.setTickPosition(QtGui.QSlider.TicksBothSides)
        self.sliderTime.setTickInterval(1)
        self.sliderTime.setFocusPolicy(QtCore.Qt.ClickFocus)

        self.sliderTime.setTracking(False)


        self.spinTime = QtGui.QSpinBox()
        self.spinTime.setStyleSheet("color:white;border:0px solid black;background-color:black;")
        self.spinTime.setButtonSymbols(QtGui.QAbstractSpinBox.NoButtons)
        self.spinTime.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.spinTime.setAttribute(QtCore.Qt.WA_MacShowFocusRect, 0)


        self.sliderTime.valueChanged.connect(self.spinTime.setValue)
        self.spinTime.valueChanged.connect(self.sliderTime.setValue)


        self.scaleSlider = FloatSlider(QtCore.Qt.Vertical)

        self.scaleSlider.setRange(self.N_SCALE_MIN_EXP,self.N_SCALE_MAX_EXP,500)
        self.scaleSlider.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.scaleSlider.setToolTip("value scale")

        self.minSlider = FloatSlider(QtCore.Qt.Vertical)
        self.minSlider.setRange(self.N_SCALE_MIN_EXP,self.N_SCALE_MAX_EXP,500)
        self.minSlider.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.minSlider.setToolTip("min scale")

        
        self.gammaSlider = FloatSlider(QtCore.Qt.Vertical)
        self.gammaSlider.setRange(.01,2.,200)

        self.gammaSlider.setToolTip("value gamma")
        self.gammaSlider.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.gammaSlider.setValue(1.)


        # def func_from_n(n):
        #     return 2**(self.N_SCALE_MIN_EXP+(self.N_SCALE_MAX_EXP-self.N_SCALE_MIN_EXP)*(n-1.)/(self.N_SCALE_SLIDER-1))


        # def func_to_n(x):
        #     if x<2**self.N_SCALE_MIN_EXP:
        #         print "gg", x
        #         return 1
        #     elif x>2**self.N_SCALE_MAX_EXP:
        #         return self.N_SCALE_SLIDER

        #     return int(round(1.+(self.N_SCALE_SLIDER-1.)*(np.log2(x)-self.N_SCALE_MIN_EXP)/(self.N_SCALE_MAX_EXP-1.*self.N_SCALE_MIN_EXP)))


        # self.scaleSlider.valueChanged.connect(lambda x: self.transform.setValueScale(0,func_from_n(x)))
        # self.transform._maxChanged.connect(lambda x: self.scaleSlider.setValue(func_to_n(x)))


        def func1(x):
            return 2**x

        def func2(x):
            return np.log2(x)

        self.scaleSlider.floatValueChanged.connect(lambda x: self.transform.setMax(func1(x)))
        self.transform._maxChanged.connect(lambda x:self.scaleSlider.setValue(func2(x)))


        self.minSlider.floatValueChanged.connect(lambda x: self.transform.setMin(func1(x)))
        self.transform._minChanged.connect(lambda x:self.minSlider.setValue(func2(x)))

        
        self.gammaSlider.floatValueChanged.connect(self.transform.setGamma)
        self.transform._gammaChanged.connect(self.gammaSlider.setValue)

        # self.keyframes = KeyFrameList()
        self.keyPanel = KeyFramePanel(self.glWidget)
        self.keyPanel.hide()

        self.impListView = ImageProcessorListView([BlurProcessor(),
                                                   BlurXYZProcessor(),
                                                   NoiseProcessor(),
                                                   FFTProcessor(),
                                                   LucyRichProcessor()])

        self.impListView.hide()

        self.volSettingsView = VolumeSettingsPanel()
        self.volSettingsView.hide()

        self.settingsView = MainSettingsPanel()
        self.settingsView.hide()

        self.setStyleSheet("""
        background-color:black;
        color:black;
        """)

        hbox0 = QtGui.QHBoxLayout()
        hbox0.addWidget(self.minSlider)

        hbox0.addWidget(self.scaleSlider)
        hbox0.addWidget(self.gammaSlider)

        hbox0.addWidget(self.glWidget,stretch = 3)

        hbox0.addWidget(self.sliceWidget,stretch =2)


        hbox0.addWidget(self.impListView)


        hbox0.addWidget(self.volSettingsView)
        hbox0.addWidget(self.settingsView)

        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.bwdButton)
        hbox.addWidget(self.startButton)
        hbox.addWidget(self.fwdButton)
        hbox.addWidget(self.sliderTime)
        hbox.addWidget(self.spinTime)

        hbox.addWidget(self.centerButton)

        hbox.addWidget(self.rotateButton)

        hbox.addWidget(self.checkKey)
        hbox.addWidget(self.screenshotButton)
        hbox.addWidget(self.fileOpenButton)
        hbox.addWidget(self.fileSaveButton)

        hbox.addSpacing(50)
        hbox.addWidget(self.checkIsoView)
        hbox.addWidget(self.checkSliceView)

        hbox.addSpacing(10)
        hbox.addWidget(self.checkProcView)

        hbox.addSpacing(10)
        hbox.addWidget(self.checkVolSettings)

        hbox.addSpacing(5)
        hbox.addWidget(self.checkSettings)

        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(hbox0)

        vbox.addLayout(hbox)
        vbox.addWidget(self.keyPanel)


        for box in [hbox,vbox,hbox0]:
            box.setContentsMargins(0,0,0,0)

        vbox.setSpacing(1)
        hbox.setSpacing(11)
        hbox0.setSpacing(5)


        self.egg3d = spimagine.utils.egg3d.Egg3dController()

        # widget = QtGui.QWidget()
        self.setLayout(vbox)



        self.rotateTimer = QtCore.QTimer(self)
        self.rotateTimer.setInterval(70)
        self.rotateTimer.timeout.connect(self.onRotateTimer)

        self.playTimer = QtCore.QTimer(self)
        self.playTimer.setInterval(100)
        self.playTimer.timeout.connect(self.onPlayTimer)
        self.settingsView._playIntervalChanged.connect(self.playIntervalChanged)
        self.setLoopBounce(True)

        self.playDir = 1

        # a decorator as checkboxe state is  2 if checked
        def stateToBool(objFunc, invert = False):
            def _foo(x):
                if invert:
                    return objFunc(x!=2)
                else:
                    return objFunc(x==2)
            return _foo

        self.volSettingsView.checkBox.stateChanged.connect(
            self.glWidget.transform.setBox)

        self.volSettingsView.checkInvert.stateChanged.connect(
            stateToBool(self.glWidget.set_background_mode_black,invert = True))

        self.settingsView._substepsChanged.connect(self.substepsChanged)

        self.checkIsoView.stateChanged.connect(
            stateToBool(self.glWidget.transform.setIso))

        self.transform._isoChanged.connect(self.checkIsoView.setChecked)


        self.settingsView.checkEgg.stateChanged.connect(self.onCheckEgg)

        self.volSettingsView._boundsChanged.connect(self.glWidget.transform.setBounds)

        self.volSettingsView.sliderAlphaPow.floatValueChanged.connect(self.glWidget.transform.setAlphaPow)

        self.volSettingsView.sliderOcc.floatValueChanged.connect(
            self.glWidget.transform.setOccStrength)

        self.volSettingsView.sliderOccRadius.floatValueChanged.connect(
                self.glWidget.transform.setOccRadius)
        self.volSettingsView.sliderOccNPoints.floatValueChanged.connect(
            self.glWidget.transform.setOccNPoints)

        self.glWidget.transform._alphaPowChanged.connect(self.volSettingsView.sliderAlphaPow.setValue)

        self.glWidget.transform._boundsChanged.connect(self.volSettingsView.setBounds)

        self.transform._boxChanged.connect(self.volSettingsView.checkBox.setChecked)


        self.volSettingsView.checkProj.stateChanged.connect(self.transform.setPerspective)
        self.transform._perspectiveChanged.connect(self.volSettingsView.checkProj.setChecked)

        self.checkKey.stateChanged.connect(self.keyPanel.setVisible)

        self.checkVolSettings.stateChanged.connect(self.volSettingsView.setVisible)

        self.checkSettings.stateChanged.connect(self.settingsView.setVisible)

        self.checkSliceView.stateChanged.connect(self.sliceWidget.setVisible)
        self.checkSliceView.stateChanged.connect(self.transform.setShowSlice)

        self.checkProcView.stateChanged.connect(self.impListView.setVisible)

        self.settingsView.checkLoopBounce.stateChanged.connect(self.setLoopBounce)

        self.volSettingsView._stackUnitsChanged.connect(self.transform.setStackUnits)
        self.transform._stackUnitsChanged.connect(self.volSettingsView.setStackUnits)

        self.settingsView._frameNumberChanged.connect(self.keyPanel.setFrameNumber)

        self.volSettingsView.colorCombo.currentIndexChanged.connect(self.onColormapChanged)
        self.volSettingsView._rgbColorChanged.connect(self.onRgbColorChanged)
        self.impListView._stateChanged.connect(self.impStateChanged)

        self.settingsView._dirNameChanged.connect(self.keyPanel.setDirName)
        # dataModel._dataSourceChanged.connect(self.dataSourceChanged)
        # dataModel._dataPosChanged.connect(self.sliderTime.setValue)

        self.glWidget._dataModelChanged.connect(self.dataModelChanged)

        self.onColormapChanged(0)

        self.checkSliceView.setChecked(False)

        self.hiddableControls = [self.checkSettings,self.checkVolSettings,
                                 self.startButton,self.sliderTime,self.spinTime,
                                 self.checkKey,self.screenshotButton ]

        # self.keyPanel.keyView.setModel(self.keyframes)
        

    def impStateChanged(self):
        data = self.transform.dataModel[self.transform.dataPos]

        for imp in self.impListView.impViews:
            if imp.is_active():
                print "active: ", imp.proc.name, imp.proc.kwargs
                data = imp.proc.apply(data)

        self.glWidget.renderer.update_data(data)
        self.glWidget.refresh()

    def onColormapChanged(self,index):
        self.glWidget.set_colormap(self.volSettingsView.colormaps[index])
        self.glWidget.refresh()

        self.sliceWidget.glSliceWidget.set_colormap(self.volSettingsView.colormaps[index])
        self.sliceWidget.glSliceWidget.refresh()


    def onRgbColorChanged(self,r,g,b):
        print r,g,b
        self.glWidget.set_colormap_rgb([r,g,b])
        
        self.glWidget.refresh()

        self.sliceWidget.glSliceWidget.set_colormap_rgb([r,g,b])
        self.sliceWidget.glSliceWidget.refresh()

    def onCheckEgg(self,state):
        if state == QtCore.Qt.Checked:
            self.connectEgg3d()
        else:
            self.egg3d.stop()

    def connectEgg3d(self):
        try:
            self.egg3d.listener._quaternionChanged.connect(self.egg3dQuaternion)
            self.egg3d.listener._zoomChanged.connect(self.egg3dZoom)

            N = 45
            self._quatHist = [Quaternion() for i in range(N)]
            self._quatWeights = np.exp(-2.*np.linspace(0,1,N))
            self._quatWeights *= 1./sum(self._quatWeights)
            self.egg3d.start()
        except Exception as e:
            print e
            self.settingsView.checkEgg.setCheckState(QtCore.Qt.Unchecked)


    def egg3dQuaternion(self,a,b,c,d):
        self._quatHist = np.roll(self._quatHist,1)
        self._quatHist[0] = Quaternion(a,c,b,d)

        q0 = Quaternion(0,0,0,0)
        for q,w in zip(self._quatHist,self._quatWeights):
            q0 = q0+q*w

        self.transform.setQuaternion(q0)

    def egg3dZoom(self,zoom):
        if zoom>0:
            newZoom = self.transform.zoom * 1.01**zoom
        else:
            newZoom = self.transform.zoom * 1.1**zoom

        newZoom = np.clip(newZoom,.7,3)
        self.transform.setZoom(newZoom)


    def initUI(self):
        pass


    def keyPressEvent(self, event):
        if type(event) == QtGui.QKeyEvent:
            if event.modifiers()== QtCore.Qt.ControlModifier and  event.key() == QtCore.Qt.Key_Q:
                return
        # super(MainWidget,self).keyPressEvent(event)

    def initActions(self):
        QtGui.QShortcut(QtGui.QKeySequence("Ctrl+W"), self, self.closeMe)

        # self.exitAction = QtGui.QAction('Quit', self)
        # self.exitAction.setShortcut('Ctrl+Q')
        # self.exitAction.setStatusTip('Exit application')
        # self.exitAction.triggered.connect(self.foo)

    def setModel(self,dataModel):

        self.glWidget.setModel(dataModel)
        self.sliceWidget.setModel(dataModel)


    def dataModelChanged(self):
        logger.debug("data Model changed")
        dataModel = self.glWidget.dataModel
        dataModel._dataSourceChanged.connect(self.dataSourceChanged)

        dataModel._dataPosChanged.connect(self.sliderTime.setValue)
        self.sliderTime.valueChanged.connect(self.transform.setPos)

        kvList = KeyFrameList()
        kvList.addItem(KeyFrame(0.))
        kvList.addItem(KeyFrame(1.))

        self.keyPanel.connect_to_transform(self.transform)
        self.keyPanel.setModel(kvList)

        self.dataSourceChanged()

    def dataSourceChanged(self):
        self.sliderTime.setRange(0,self.glWidget.dataModel.sizeT()-1)
        self.sliderTime.setValue(0)
        self.spinTime.setRange(0,self.glWidget.dataModel.sizeT()-1)

        self.volSettingsView.dimensionLabel.setText("Dim: %s"%str(tuple(self.glWidget.dataModel.size()[::-1])))


        if self.myparent:
            self.myparent.setWindowTitle(self.glWidget.dataModel.name())
        else:
            self.setWindowTitle(self.glWidget.dataModel.name())

        self.keyPanel.connect_to_transform(self.transform)


        d = self.glWidget.dataModel[self.glWidget.dataModel.pos]
        # minMaxMean = (np.amin(d),np.amax(d),np.mean(d))
        # self.volSettingsView.statsLabel.setText("Min:\t%.2f\nMax:\t%.2f \nMean:\t%.2f"%minMaxMean)




    def forward(self,event):
        if self.glWidget.dataModel:
            newpos = (self.glWidget.dataModel.pos+1)%self.glWidget.dataModel.sizeT()
            self.transform.setPos(newpos)

    def backward(self,event):
        if self.glWidget.dataModel:
            newpos = (self.glWidget.dataModel.pos-1)%self.glWidget.dataModel.sizeT()
            self.transform.setPos(newpos)


    def startPlay(self,event):
        if self.playTimer.isActive():
            self.playTimer.stop()
            self.startButton.setIcon(QtGui.QIcon(absPath("images/icon_start.png")))

        else:
            self.playTimer.start()
            self.startButton.setIcon(QtGui.QIcon(absPath("images/icon_pause.png")))


    def screenShot(self):
        fileName = QtGui.QFileDialog.getSaveFileName(self, 'Save screenshot as',
                                                     '.', selectedFilter='*.png')

        if fileName:
            self.glWidget.saveFrame(str(fileName))


    def set_colormap(self, name):
        self.glWidget.set_colormap(name)

    def set_colormap_rgb(self, color):
        self.glWidget.set_colormap_rgb(color)


    def saveFrame(self, fName):
        self.glWidget.saveFrame(fName)



    def setLoopBounce(self,loopBounce):
        #if loopBounce = True, then while playing it should loop back and forth
        self.loopBounce = loopBounce
        self.settingsView.checkLoopBounce.setChecked(loopBounce)


    def playIntervalChanged(self,val):
        if self.playTimer.isActive():
            self.playTimer.stop()
        self.playTimer.setInterval(val)

    def substepsChanged(self,val):
        self.glWidget.NSubrenderSteps = val
        self.glWidget.refresh()

    def openFile(self,e):
        # path = QtGui.QFileDialog.getOpenFileName(self, 'Open Path (File or Folder)',
        #                                              '.', selectedFilter='*.tif')


        f = QtGui.QFileDialog()
        f.setWindowTitle('Open Path (File or Folder)')
        f.setFileMode(QtGui.QFileDialog.ExistingFile & QtGui.QFileDialog.Directory)
        f.exec_()
        path = f.selectedFiles()

        if len(path)==0:
            return

        path = str(path[0])
        print path
        if path:
            try:
                if self.glWidget.dataModel:
                    self.glWidget.dataModel.loadFromPath(path,
                        prefetchSize = self.glWidget.N_PREFETCH)
                else:
                    self.glWidget.setModel(DataModel.fromPath(path,
                        prefetchSize = self.glWidget.N_PREFETCH))
            except Exception as e:
                mbox = QtGui.QMessageBox()
                mbox.setText(str(e))
                mbox.setIcon(QtGui.QMessageBox.Warning)
                mbox.exec_()

    def saveFile(self,e):
        path = QtGui.QFileDialog.getSaveFileName(self, 'Save as Tif File',
                                                     '.', selectedFilter='*.tif')

        path = str(path)
        
        if path:
            if self.glWidget.dataModel:
                write3dTiff(self.glWidget.dataModel[self.transform.dataPos].astype(np.float32),path)

    def onPlayTimer(self):

        if self.glWidget.dataModel:


            if self.glWidget.dataModel.pos == self.glWidget.dataModel.sizeT()-1:
                self.playDir = 1-2*self.loopBounce
            if self.glWidget.dataModel.pos == 0:
                self.playDir = 1

            newpos = (self.glWidget.dataModel.pos+self.playDir)%self.glWidget.dataModel.sizeT()
            self.transform.setPos(newpos)


    def contextMenuEvent(self,event):
         # create context menu
        popMenu = QtGui.QMenu(self)
        action = QtGui.QAction('toggle controls', self)
        action.triggered.connect(self.toggleControls)
        popMenu.addAction(action)
        popMenu.setStyleSheet("background-color: white")
        # popMenu.exec_(QtGui.QCursor.pos())


    def toggleControls(self):
        for c in self.hiddableControls:
            c.setVisible(not c.isVisible())

    def closeMe(self):
        #little workaround as on MAC ctrl-q cannot be overwritten

        self.isCloseFlag = True
        self.close()

    def closeEvent(self,event):
        logger.debug("closeevent")
        if not event.spontaneous() and not self.isCloseFlag:
            event.ignore()
        else:
            if self.playTimer.isActive():
                self.playTimer.stop()

            if self.rotateTimer.isActive():
                self.rotateTimer.stop()
                
            self.glWidget.renderTimer.stop()
            
            # self.glWidget.setParent(None)
            # free the gpu resources....
            logger.debug("deleting the renderer")
            del self.glWidget.renderer
            event.accept()


    def center(self):
        self.transform.center()

    def rotate(self):
        if self.rotateTimer.isActive():
            self.rotateTimer.stop()
            self.rotateButton.setIcon(QtGui.QIcon(absPath("images/icon_rotate.png")))

        else:
            self.rotateTimer.start()
            self.rotateButton.setIcon(QtGui.QIcon(absPath("images/icon_rotate_active.png")))


    def onRotateTimer(self):
        self.transform.addRotation(-.02,0,1.,0)
        self.glWidget.render()
        self.glWidget.updateGL()

    def mouseDoubleClickEvent(self,event):
        super(MainWidget,self).mouseDoubleClickEvent(event)
        if self.isFullScreen:
            self.showNormal()
        else:
            self.showFullScreen()

        self.glWidget.resized = True

        # there's a bug in Qt that disables drop after fullscreen, so reset it...
        self.setAcceptDrops(True)

        self.isFullScreen = not self.isFullScreen


def test_sphere():
    from spimagine import DataModel, NumpyData

    x = np.linspace(-1,1,128)
    Z,Y,X = np.meshgrid(x,x,x)
    # R = sqrt(Z**2+Y**2+(X-.35)**2)
    # R2 = sqrt(Z**2+Y**2+(X+.35)**2)

    # d = 100.*exp(-10*R**2)+.0*np.random.normal(0,1.,X.shape)

    # d += 100.*exp(-10*R2**2)+.0*np.random.normal(0,1.,X.shape)

    Ns = 5
    r = .6
    phi = np.linspace(0,2*np.pi,Ns+1)[:-1]
    d = np.zeros_like(X)
    for p in phi:
        d += 200.*np.exp(-10*(Z**2+(Y-r*np.sin(p))**2+(X-r*np.cos(p))**2))


    app = QtGui.QApplication(sys.argv)

    win = MainWidget()

    win.setModel(DataModel(NumpyData(d)))

    # win.glWidget.transform.setIso(True)
    # win.glWidget.transform.setValueScale(0,40)

    # win.saveFile(None)
    win.show()

    win.raise_()

    sys.exit(app.exec_())


def test_surface():
    from spimagine import DataModel, NumpyData, DemoData
    from spimagine.gui.mesh import SphericalMesh

    app = QtGui.QApplication(sys.argv)

    win = MainWidget()


    x = np.linspace(-1,1,128)
    Z,Y,X = np.meshgrid(x,x,x)
    d = np.exp(-10*X**2)



    #win.setModel(DataModel(NumpyData(d)))
    win.setModel(DataModel(DemoData()))

    # win.glWidget.add_surface_ellipsoid((1.,0,0),
    #                                    (.2,.2,.2),
    #                                    facecolor = (1.,.3,.1,.5),
    #                                    edgecolor = (1.,1.,1.,.2),
    #                                    Nphi =20, Ntheta=10)

    # win.glWidget.add_mesh(SphericalMesh(facecolor = (1.,1.,1.),
    #                            edgecolor = None))


    win.show()

    win.raise_()


    sys.exit(app.exec_())



if __name__ == '__main__':
    #test_sphere()

    test_surface()



"""
Implements generic classes for processing of volumetric data
"""

import sys
import numpy as np
import gputools



# try:
#     import imgtools
#     import lucy_richardson_gpu
# except:
#     print "could not import imgtools or lucy_richardson module"

class ImageProcessor(object):
    def __init__(self,name = "",**kwargs):
        self.name = name
        self.set_params(**kwargs)

    def set_params(self,**kwargs):
        self.kwargs = kwargs

    def apply(self, data):
        raise NotImplementedError()

    def __getattr__(self,attr):
        if self.kwargs.has_key(attr):
            return self.kwargs[attr]
        else:
            return super(ImageProcessor,self).__getattr__(attr)



class CopyProcessor(ImageProcessor):

    def __init__(self):
        super(CopyProcessor,self).__init__("copy")

    def apply(self,data):
        return data


class BlurProcessor(ImageProcessor):

    def __init__(self,sigma = 4.):
        super(BlurProcessor,self).__init__("blur",sigma = sigma)

    def apply(self,data):
        N  = 2*self.sigma+1
        x = np.arange(-N,N+1)
        h = np.exp(-x**2/2./self.sigma**2)
        h *= 1./sum(h)
        return gputools.convolve_sep3(data, h, h, h)


class BlurXYZProcessor(ImageProcessor):

    def __init__(self,sx=4.,sy=4.,sz=4.):
        super(BlurXYZProcessor,self).__init__("blur_xyz",sx=sx,sy=sy,sz=sz)

    def apply(self,data):
        sigs = (self.sx,self.sy,self.sz)
        Ns  = [2*s+1 for s in sigs]

        xs = [np.arange(-N,N+1) for N in Ns]
        hs = [np.exp(-x**2/2./s**2) for x,s in zip(xs,sigs)]
        hs = [1.*h/sum(h) for h in hs]
        return gputools.convolve_sep3(data, *hs)

class NoiseProcessor(ImageProcessor):
    def __init__(self,sigma = 10):
        super(NoiseProcessor,self).__init__("noise",sigma = sigma)

    def apply(self,data):
        return np.maximum(0,data+self.sigma*np.random.normal(0,1,data.shape))


class FFTProcessor(ImageProcessor):
    def __init__(self, log = False):
        super(FFTProcessor,self).__init__("fft", log = log)
        self.log = log

    def apply(self,data):
        dshape = data.shape
        
        res = gputools.pad_to_power2(data.astype(np.complex64), mode = "wrap")

        res = 1./np.sqrt(res.size)*np.fft.fftshift(abs(gputools.fft(res)))
        
        res = gputools.pad_to_shape(res,dshape)
        
        if self.log:
            return np.log2(0.001+res)
        else:
            return res


class LucyRichProcessor(ImageProcessor):

    def __init__(self,rad = 4., niter = 6):
        super(LucyRichProcessor,self).__init__("RL-Deconv",rad = rad, niter = niter)
        self.rad0 = rad
        self.niter0 = niter
        self.hshape = (1,)*3

    def reset_psf(self,dshape):
        # self.h = imgtools.blur_psf(dshape,self.rad)
        pass

    def apply(self,data):
        if self.hshape != data.shape or self.rad != self.rad0:
            self.reset_psf(data.shape)
            self.rad0 = self.rad

        return data
        # return lucy_richardson_gpu.lucy_richardson(data, self.h,self.niter)


class FuncProcessor(ImageProcessor):

    def __init__(self,func, name = "func processor", **kwargs):
        super(FuncProcessor,self).__init__(name,**kwargs)
        self.func = func

    def apply(self,data):
        return self.func(data,**self.kwargs)


    
if __name__ == '__main__':
    from numpy import *


    # p = LucyRichProcessor()

    # p2 = FFTProcessor()

    # print p.name

    # print p2.name








import logging
logger = logging.getLogger(__name__)



import numpy as np
import bisect
from PyQt4 import QtCore
import json
import sortedcontainers

from spimagine.utils.quaternion import *

import spimagine

def create_interp_func(a):
    """ an elastic interpolation function,
    the bigger a, the more elastic 
    when a==0, its the usual hard linear interpolation 
    """
    
    def atan_func(x):
        return .5*(1+np.arctan(2*a*(x-.5))/np.arctan(a))

    def linear_func(x):
        return x

    if a==0:
        return linear_func
    else:
        return atan_func


class KeyableParameter(object):
    def __init__(self):
        self.val_dict = {}

    def register_value(self,name, default = 0.):
        self.val_dict[name] = default

    def __getattr__(self,name):
        return self.val_dict[name]



class TransformData(object):
    def __init__(self,quatRot = Quaternion(),
                 zoom = 1,
                 dataPos = 0,
                 minVal = 0.,
                 maxVal = 100.,
                 gamma = 1.,
                 translate = [0,0,0],
                 bounds = [-1,1,-1,1,-1,1],
                 isBox = True,
                 isIso = False,
                 alphaPow = 0.):
        self.setData(quatRot=quatRot,
                     zoom = zoom,
                     dataPos= dataPos,
                     minVal = minVal,
                     maxVal = maxVal,
                     gamma= gamma,
                     translate = translate,
                     bounds = bounds,
                     isBox = isBox,
                     isIso = isIso,
                     alphaPow = alphaPow)


    def __repr__(self):
        return """TransformData:
        quatRot\t %s
        translate\t %s
        zoom\t%s
        datapos\t%s
        minVal\t%s
        maxVal\t%s
        gamma\t%s
        bounds\t%s
        isBox\t%s
        isIso\t%s
        alphaPow\t%s"""%(str(self.quatRot),str(self.translate),self.zoom,self.dataPos,self.minVal,self.maxVal, self.gamma, self.bounds, self.isBox,self.isIso, self.alphaPow)

        

    def setData(self,quatRot,zoom, dataPos, minVal, maxVal,
                gamma, translate, bounds,isBox,isIso, alphaPow):

        self.quatRot = Quaternion.copy(quatRot)
        self.zoom = zoom
        self.dataPos = dataPos
        self.minVal = minVal

        self.maxVal = maxVal
        self.gamma = gamma
        self.bounds  = np.array(bounds)
        self.isBox = isBox
        self.isIso = isIso
        self.alphaPow = alphaPow
        self.translate = np.array(translate)

    @classmethod
    def interp(cls,x1, x2, lam ,f = create_interp_func(0.)):
        """
        f should be a function [0...1] - [0...1]
        """
        t = f(lam)
        newQuat = quaternion_slerp(x1.quatRot, x2.quatRot, t)
        newZoom = (1.-t)*x1.zoom + t*x2.zoom
        newPos = int((1.-t)*x1.dataPos + t*x2.dataPos)
        newMinVal = (1.-t)*x1.minVal + t*x2.minVal

        newMaxVal = (1.-t)*x1.maxVal + t*x2.maxVal
        newGamma = (1.-t)*x1.gamma + t*x2.gamma

        newBounds = (1.-t)*x1.bounds + t*x2.bounds

        newAlphaPow = (1.-t)*x1.alphaPow + t*x2.alphaPow
        newTranslate = (1.-t)*x1.translate + t*x2.translate

        # some things should not be interpolated...
        newBox = x1.isBox
        newIso = x1.isIso

        return TransformData(quatRot = newQuat,zoom = newZoom,
                             dataPos= newPos,
                             minVal = newMinVal,
                             maxVal = newMaxVal,
                             gamma= newGamma,
                             translate = newTranslate,
                             bounds= newBounds,
                             isBox = newBox,
                             isIso = newIso,
                             alphaPow = newAlphaPow)



class KeyFrame(object):
    def __init__(self,pos = 0, transformData = TransformData()):
        self.pos = pos
        self.transformData = transformData


    def __repr__(self):
        return "Keyframe at t = %.3f \n%s"%(self.pos,self.transformData)


""" the keyframe model """

class KeyFrameList(QtCore.QObject):

    _modelChanged = QtCore.pyqtSignal()
    _itemChanged = QtCore.pyqtSignal(int)

    def __init__(self):
        super(KeyFrameList,self).__init__()
        self._countID = 0
        self.posdict = sortedcontainers.SortedDict()
        self.items = dict()
        # self.addItem(KeyFrame(0.))
        # self.addItem(KeyFrame(1.))
        self._modelChanged.emit()


    def __repr__(self):
        s = "\n".join([str(self.items[ID]) for ID in self.posdict.values()])
        s += "\n%s\n%s\n"%(str(self.posdict),str(self.items.keys()))
        return s

    def dump_to_file(self,fName):
        print json.dumps(self._to_dict)

    def addItem(self, frame = KeyFrame()):
        logger.debug("KeyFrameList.addItem: %s"%frame)

        newID = self._getNewID()
        if newID in self.items:
            raise KeyError()

        if newID in self.posdict.values():
            raise KeyError()

        self.items[newID] = frame
        self.posdict[frame.pos] = newID

        # print "AFTER"
        # print self

        self._modelChanged.emit()

    def removeItem(self, ID):
        logger.debug("KeyFrameList.removeItem: %s"%ID)
        # print "REMOVE\nBEFORE"
        # print self

        self.posdict.pop(self.posdict.keys()[self.posdict.values().index(ID)])
        self.items.pop(ID)

        # print "AFTER"
        # print self

        self._modelChanged.emit()

    def __getitem__(self, ID):
        return self.items[ID]

    def _getNewID(self):
        newID = self._countID
        self._countID += 1
        return newID

    def item_at(self, index):
        """"returns keyframe in order 0...len(items)"""
        return self.items[self.item_id_at(index)]

    def item_id_at(self, index):
        """"returns keyframe id in order 0...len(items)"""
        return self.posdict.values()[index]

    def pos_at(self, index):
        return self.posdict.keys()[index]

    def pos_at_id(self, ID):
        return self.posdict.keys()[self.posdict.values().index(ID)]

    def update_pos(self,ID, pos):
        if pos in self.posdict.keys():
            print "pos already there:", pos
            return
        frame = self.items[ID]

        self.posdict.pop(self.pos_at_id(ID))
        frame.pos = pos
        self.posdict[pos] = ID

    def distribute(self,pos_start,pos_end):
        """distributes the internal data positions linearly according to their nodes position"""
        for it in self.items.values():
            print it.pos, it.transformData.dataPos 
            it.transformData.dataPos = int(pos_start+(pos_end-pos_start)*it.pos)
            # it.transformData.dataPos = pos_start+(pos_end-pos_start)*it.pos
            
    def getTransform(self,pos):
        logger.debug("getTransform")

        if pos<self.pos_at(0):
            return self.item_at(0).transformData
        if pos>self.pos_at(-1):
            return self.item_at(-1).transformData

        if self.posdict.has_key(pos):
            return self.items[self.posdict[pos]].transformData

        ind = self.posdict.bisect(pos)
        frameLeft, frameRight = self.item_at(ind-1),self.item_at(ind)

        # linear interpolating
        if np.abs(frameRight.pos-frameLeft.pos)<1.e-7:
            lam = 0.
        else:
            lam = (1.*pos-frameLeft.pos)/(frameRight.pos-frameLeft.pos)

        newTrans = TransformData.interp(frameLeft.transformData,frameRight.transformData,lam)

        return newTrans


    def _to_JSON(self):
        return json.dumps(self,cls = KeyFrameEncoder)

    @classmethod
    def _from_JSON(self,jsonStr):
        return json.loads(jsonStr,cls = KeyFrameDecoder)



"""JSON routines to save and load from file"""

class KeyFrameEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.ndarray) and obj.ndim ==1:
            return obj.tolist()

        elif isinstance(obj, Quaternion):
            return obj.data.tolist()

        elif isinstance(obj, sortedcontainers.SortedDict):
            return dict(obj)

        elif isinstance(obj, (
                KeyFrame,
                KeyFrameList,
                TransformData,
                spimagine.models.transform_model.TransformData)):
            return obj.__dict__

        elif isinstance(obj,np.generic):
            return np.asscalar(obj)

        
        return json.JSONEncoder.default(self, obj)



"""
OLD

    {"keyDict": {"1": {"transformData": {"dataPos": 0, "quatRot": [1.0, 0.0, 0.0, 0.0], "zoom": 1, "bounds": [-1, 1, -1, 1, -1, 1]}, "tFrame": 0}, "2": {"transformData": {"dataPos": 0, "quatRot": [1.0, 0.0, 0.0, 0.0], "isBox":true,"zoom": 1, "bounds": [-1, 1, -1, 1, -1, 1]}, "tFrame": 1}}, "_countID": 2, "tFrames": [[0, 1], [1, 2]]}


NEW

{"items": {"0": {"transformData": {"dataPos": 0, "quatRot": [1.0, 0.0, 0.0, 0.0], "maxVal": 100.0, "isIso": false, "minVal": 0.0, "zoom": 1, "bounds": [-1, 1, -1, 1, -1, 1], "isBox": true, "alphaPow": 0.0, "translate": [0, 0, 0], "gamma": 1.0}, "pos": 0}}, "_countID": 1, "posdict": {"0": 0}}
"""


class KeyFrameDecoder(json.JSONDecoder):
    def decode(self, s, classname = ""):
        # print classname," XXXX\n" ,s
        if classname == "":

            dec = json.JSONDecoder.decode(self,s)
            ret = KeyFrameList()
            ret._countID = dec["_countID"]
            ret.posdict = self.decode(dec["posdict"],"posdict")
            ret.items = self.decode(dec["items"],"items")
            return ret

        elif classname == "posdict":
            return sortedcontainers.SortedDict((float(k),int(v)) for k,v in s.iteritems())

        elif classname == "items":
            return dict((int(k),KeyFrame(v["pos"],self.decode(v["transformData"],"transformData"))) for k,v in s.iteritems())

        elif classname == "transformData":
            t =  TransformData()
            t.__dict__.update(s)

            t.quatRot = Quaternion(*t.quatRot)
            t.bounds = np.array(t.bounds)
            t.translate = np.array(t.translate)

            return t


def test_interpolation():
    k = KeyFrameList()
    k.addItem(KeyFrame(.5,TransformData(quatRot=Quaternion(.71,.71,0,0))))

    for t in np.linspace(0,1,10):
        print t, k.getTransform(t)


def test_shuffle():
    k = KeyFrameList()

    k.addItem(KeyFrame(.5,TransformData(zoom=.5,quatRot = Quaternion(.71,.71,0,0),bounds=[0]*6)))


    # print k
    # for t in np.linspace(-.1,1.2,11):
    #     print t, k.getTransform(t).zoom

    np.random.seed(0)

    def rand_ID(k):
        #just pick from within the border...
        return k.item_id_at(np.random.randint(1,len(k.items)-1))

    #adding some
    for i in range(4):
        k.addItem(KeyFrame(np.random.uniform(0,1)))

    print k
    #shuffle them
    for i in range(100):
        print "+++++++++++++++++"
        k.addItem(KeyFrame(np.random.uniform(0,1)))

        ID = rand_ID(k)
        print "moving %s"%ID
        k.update_pos(ID,np.random.uniform(0,1.))

        ID = rand_ID(k)
        print "removing %s"%ID
        k.removeItem(ID)

if __name__ == '__main__':

    k = KeyFrameList()

    k.addItem(KeyFrame(0,TransformData()))

    k.addItem(KeyFrame(1.,TransformData()))

    k.addItem(KeyFrame(.5,TransformData(zoom=.5,quatRot = Quaternion(.71,.71,0,0),bounds=[0]*6)))


    s = k._to_JSON()

    print s

    k2 = KeyFrameList._from_JSON(s)


    print k2.getTransform(.1)
    

    k3 = KeyFrameList._from_JSON(open("test.json","r").read())




import sys
import os
import re
import numpy as np
from scipy.misc import imread


def _absPath(myPath):
    """ Get absolute path to resource, works for dev and for PyInstaller """
    import sys

    try:
        # PyInstaller creates a temp folder and stores path in _MEIPASS
        base_path = sys._MEIPASS
        logger.DEBUG("found MEIPASS: %s "%os.path.join(base_path, os.path.basename(myPath)))

        return os.path.join(base_path, os.path.basename(myPath))
    except Exception:
        base_path = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(base_path, myPath)

def _arrayFromImage(fName):
    """converts png image to float32 array
    returns an array of shape [h,w,3]
    """
    img = imread(fName)
    # img = np.asarray(Image.open(fName).convert("RGB"))

    if len(img.shape)<3:
        raise TypeError("image %s appears not to be a 2d rgb image"%fName)

    return 1./255*img[:,:,:3]

# def _arrayFromImage_Qt(fName):
#     """converts png image to float32 array
#     returns an array of shape [w,h,3]
#     """
#     try:
#         img = QtGui.QImage(fName).convertToFormat(QtGui.QImage.Format_RGB32)
#         Nx, Ny = img.width(),img.height()
#         tmp = img.bits().asstring(img.numBytes())
#         arr = np.frombuffer(tmp, np.uint8).reshape((Ny,Nx,4))
#         arr = arr.astype(np.float32)/np.amax(arr)
#         return arr[:,:,:-1][:,:,::-1]
#     except Exception as e:
#         print e
#         print "could not load image %s"%fName
#         return np.zeros((10,100,3),np.float32)

    
def loadcolormaps():
    cmaps = {}

    try:
        basePath = sys._MEIPASS
    except:
        basePath = _absPath("../colormaps/")

    reg = re.compile("cmap_(.*)\.png")
    for fName in os.listdir(basePath):
        match = reg.match(fName)
        if match:
            try:
                cmaps[match.group(1)] = _arrayFromImage(os.path.join(basePath,fName))[0,:,:]
            except Exception as e:
                print e
                print "could not load %s"%fName

    return cmaps


if __name__ == '__main__':
    from time import time

    t = time()
    
    d =  loadcolormaps()

    print "time to load cmaps: %s s"%(time()-t)

    print len(d)



from PyQt4 import QtGui
import os


class MultipleFileDialog(QtGui.QFileDialog):
    def __init__(self, *args):
        QtGui.QFileDialog.__init__(self, *args)
        self.setOption(self.DontUseNativeDialog, False)
        self.setFileMode(self.ExistingFiles)
        btns = self.findChildren(QtGui.QPushButton)
        self.openBtn = [x for x in btns if 'open' in str(x.text()).lower()][0]
        self.openBtn.clicked.disconnect()
        self.openBtn.clicked.connect(self.openClicked)
        self.tree = self.findChild(QtGui.QTreeView)

    def openClicked(self):
        inds = self.tree.selectionModel().selectedIndexes()
        files = []
        for i in inds:
            if i.column() == 0:
                files.append(os.path.join(str(self.directory().absolutePath()),str(i.data().toString())))
        self.selectedFiles = files
        self.hide()

    def filesSelected(self):
        return self.selectedFiles


if __name__ == '__main__':
    import sys

    app = QtGui.QApplication(sys.argv)

    win = MultipleFileDialog()

    win.exec_()

    # path = QtGui.QFileDialog.getOpenFileNames(None, 'Open Tif File',
    #                                           '.', selectedFilter='*.tif')




#!/usr/bin/env python

"""
command line rendering program, currently supports just 3d tiff files

for all the options run
python spim_render.py -h



author: Martin Weigert
email: mweigert@mpi-cbg.de
"""



import sys
import argparse

from volume_render import *
from scipy.misc import toimage
from imgutils import read3dTiff, fromSpimFolder

from PIL import Image

def getTiffSize(fName):
    img = Image.open(fName, 'r')
    depth = 0
    while True:
        try:
            img.seek(depth)
        except EOFError:
            break
        depth += 1

    return (depth,)+img.size[::-1]



# def read3dTiff(fName, depth = -1, dtype = np.uint16):

#     if not depth>0:
#         depth = getTiffSize(fName)[0]

#     img = Image.open(fName, 'r')
#     stackSize = (depth,) + img.size[::-1]

#     data = np.empty(stackSize,dtype=dtype)

#     for i in range(stackSize[0]):
#         img.seek(i)
#         data[i,...] = np.asarray(img, dtype= dtype)

#     return data


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter,
        description="""renders max projectios of 3d data

    example usage:

    Tif data: \t \tspim_render  -i mydata.tif -o myoutput.png -t 0 0 -4 -u 1 1 4
    Bscope data:  \tspim_render -f bscope -i mydataFolder -o myoutput.png -t 0 0 -4 -u 1 1 4
    """)

    parser.add_argument("-f","--format",dest="format",metavar="format",
                        help = """format currently supported:
    tif (default)
    bscope  """,
                        type=str,default = "tif", required = False)

    parser.add_argument("-i","--input",dest="input",metavar="infile",
                        help = "name of the input file to render, currently only 3d Tiff is supported",
                        type=str,default = None, required = True)

    parser.add_argument("-o","--output",dest="output",metavar="outfile",
                        help = "name of the output file,  png extension is recommended",
                        type=str,default = "out.png")

    parser.add_argument("-p","--pos",dest="pos",metavar="timepoint position",
                        help = "timepoint to render if format=='bscope' ",
                        type=int,default = 0)

    parser.add_argument("-w","--width",dest="width",metavar="width",
                        help = "pixelwidth of the rendered output ",
                        type=int,default = 400)

    parser.add_argument("-s","--scale",dest="scale",metavar="scale",
                        type=float,nargs=1 ,default = [1.])


    parser.add_argument("-u","--units",dest="units",metavar="units",
                        type=float,nargs= 3 ,default = [1.,1.,5.])


    parser.add_argument("-t","--translate",dest="translate",
                        type = float, nargs=3,default = [0,0,-4],
                        metavar=("x","y","z"))

    parser.add_argument("-r","--rotation",dest="rotation", type =
                        float, nargs=4,default = [0,1,0,0],
                        metavar=("w","x","y","z"))

    parser.add_argument("-R","--range",dest="range", type =
                        float, nargs=2,default = None,
                        help = "if --16bit is set, the range of the data values to consider, defaults to [min,max]",
                        metavar=("min","max"))


    parser.add_argument("-O","--Orthoview",help="use parallel projection (default: perspective)",
                        dest="ortho",action="store_true")

    parser.add_argument("--16bit",help="render into 16 bit png",
                        dest="is16Bit",action="store_true")

    if len(sys.argv)==1:
        parser.print_help()
        return

    args = parser.parse_args()


    for k,v in vars(args).iteritems():
        print k,v

    rend = VolumeRenderer((args.width,args.width))

    if args.format=="tif":
        data = read3dTiff(args.input)
    elif args.format=="bscope":
        data = fromSpimFolder(args.input,pos=args.pos,count=1)[0,...]
    else:
        print "ERROR: format", args.format, "not supported (should be tif/bscope)"
        return

    rend.set_data(data)
    rend.set_units(args.units)

    M = scaleMat(*(args.scale*3))
    M = dot(rotMat(*args.rotation),M)
    M = dot(transMatReal(*args.translate),M)

    rend.set_modelView(M)

    if args.ortho:
        rend.set_projection(projMatOrtho(-1,1,-1,1,-1,1))
    else:
        rend.set_projection(projMatPerspective(60,1.,1,10))

    out = rend.render()

    # image is saved by scipy.misc.toimage(out,low,high,cmin,cmax)
    # p' = p * high/cmax

    if not args.is16Bit:
        imsave(args.output,out)
        # if not args.range:
        #     imsave(args.output,out)
        # else:
        #     img = toimage(out, low = args.range[0], high  = args.range[1])
        #     img.save(args.output)

    else:
        if not args.range:
            print  "min/max: ", amin(out), amax(out)
            img = toimage(out, low = amin(out), high  = amax(out),mode = "I")
        else:
            img = toimage(out, low = args.range[0], high  = args.range[1], mode = "I")
        img.save(args.output)


if __name__ == '__main__':
    main()



"""

mweigert@mpi-cbg.de
"""

import numpy as np
from spimagine.volumerender.volume_render import VolumeRenderer
import pylab
import logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

def test_simple_rendering():
    from gputools.utils.utils import remove_cache_dir, get_cache_dir
    remove_cache_dir()

    dtypes = [np.float32, np.uint16]

    # build some test data
    N = 128
    x = np.linspace(-1,1,N)
    Z,Y,X = np.meshgrid(x,x,x,indexing="ij")
    R1 = np.sqrt((X-.2)**2+Y**2+Z**2)
    R2 = np.sqrt((X+.2)**2+Y**2+Z**2)
    data = 255*np.exp(-30*R1**2)+ np.exp(-30*R2**2)

    rend = VolumeRenderer((400,400))
    outs = []

    for dtype in dtypes:
        rend.render(data=data.astype(dtype), maxVal = 255.)
        outs.append(rend.output)

    # drawing
    #pylab.ioff()
    pylab.figure(1)
    pylab.clf()
    for i,out in enumerate(outs):
        pylab.subplot(1,len(outs),i+1)
        pylab.imshow(out)
        pylab.axis("off")
        pylab.title("%s"%(dtype))

    pylab.show()
    pylab.pause(2)
    return rend

def test_surface():
    from spimagine.utils.transform_matrices import *
    N = 128
    x = np.linspace(-1,1,N)
    Z,Y,X = np.meshgrid(x,x,x,indexing="ij")
    R = np.sqrt(X**2+Y**2+Z**2)

    data = 900.*np.exp(-10*R)
    #data = 200*(1+Z)

    rend = VolumeRenderer((400,400))


    rend.set_modelView(mat4_translate(0,0,-5.))
    rend.render(data=data.astype(np.uint16), maxVal = 20., method="iso_surface")
    #rend.render(data=data.astype(np.float32), maxVal = 100., method="max_project")

    # drawing
    #pylab.ioff()
    pylab.figure(1)
    pylab.clf()

    pylab.imshow(rend.output)
    pylab.axis("off")

    pylab.show()
    pylab.pause(.1)
    return rend


if __name__ == "__main__":
    from gputools import remove_cache_dir

    remove_cache_dir()
    #rend = test_simple_rendering()
    rend = test_surface()
    out = rend.output
